@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.h,v 1.24 2004/01/15 09:31:08 fuyu Exp $
+ * $Id: bbs_thread_view.h,v 1.25 2004/01/19 00:38:47 fuyu Exp $
  */
 
 #ifndef _BBS_THREAD_VIEW_H_
@@ -166,7 +166,8 @@ void bbs_thread_view_append_text_as_sage(BBSThreadView *view,
 void bbs_thread_view_append_text_as_hidden_link(BBSThreadView *view,
 						const gchar *text, int len,
 						const gchar *link);
-void bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget);
+void bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget,
+				   int hborder, int vborder);
 void bbs_thread_view_insert_widget_at_offset(BBSThreadView *view,
 					     GtkWidget *widget,
 					     int offset);
