@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.83 2004/01/27 15:02:40 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.86 2004/02/28 22:57:33 fuyu Exp $
  */
 
 #include "config.h"
@@ -215,11 +215,6 @@ open_threadlist_view_cb(GtkWidget *selector, OchushaBulletinBoard *board,
 	ochusha_open_url(application,
 			 ochusha_bulletin_board_get_base_url(board),
 			 in_tab, with_browser);
-#if 0	/* Թΰ */
-      else
-	fprintf(stderr, "open_threadlist_view_cb: %s\n",
-		ochusha_bulletin_board_get_base_url(board));
-#endif
     }
 }
 
@@ -1771,7 +1766,10 @@ kill_board_cb(BoardlistView *view, OchushaBoardCategory *category,
       if (IS_VIRTUAL_BOARD(board))
 	{
 	  BoardCategoryGUIInfo *info = ensure_board_category_info(category);
-	  if (info->category_type == CATEGORY_TYPE_OCHUSHA_MY_FOLDER)
+	  const char *base_url = ochusha_bulletin_board_get_base_url(board);
+	  if (info->category_type == CATEGORY_TYPE_OCHUSHA_MY_FOLDER
+	      && (strcmp(base_url, "http:///favorites") == 0
+		  || strcmp(base_url, "http:///all-threads") == 0))
 	    return FALSE;
 	}
       ochusha_board_category_remove_board(category, board);
@@ -2111,7 +2109,8 @@ new_thread_dialog_response_cb(GtkWidget *dialog, int response_id,
 	  OCHU_THREADS_LEAVE();
 	  result = ochusha_bulletin_board_create_new_thread(board,
 						application->broker,
-						title, &response);
+						title, &response,
+						application->dolib_use_id_for_posting);
 	  OCHU_THREADS_ENTER();
 	  g_free(message);
 	  if (result)
