@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.32 2004/01/16 08:01:46 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.33 2004/01/27 15:49:11 fuyu Exp $
  */
 
 #include "config.h"
@@ -1466,6 +1466,9 @@ boardlist_view_button_release(GtkWidget *widget, GdkEventButton *event)
 {
   BoardlistView *view = (BoardlistView *)widget;
   GtkTreePath *path = NULL;
+  gboolean check_expander = FALSE;
+  gboolean expanded = FALSE;
+  gboolean result;
 
   g_return_val_if_fail(IS_BOARDLIST_VIEW(view), FALSE);
   g_return_val_if_fail(event != NULL, FALSE);
@@ -1494,10 +1497,12 @@ boardlist_view_button_release(GtkWidget *widget, GdkEventButton *event)
       switch (row_type)
 	{
 	case BOARDLIST_CATEGORY_ROW:
+	  expanded = gtk_tree_view_row_expanded(&view->view, path);
 	  g_signal_emit(G_OBJECT(view),
 			boardlist_view_signals[SELECT_CATEGORY_SIGNAL],
 			0,
 			data);
+	  check_expander = TRUE;
 	  break;
 
 	case BOARDLIST_BOARD_ROW:
@@ -1516,9 +1521,6 @@ boardlist_view_button_release(GtkWidget *widget, GdkEventButton *event)
     }
 
  done:
-  if (path != NULL)
-    gtk_tree_path_free(path);
-
   if (view->last_event_path != NULL)
     {
       gtk_tree_path_free(view->last_event_path);
@@ -1526,9 +1528,26 @@ boardlist_view_button_release(GtkWidget *widget, GdkEventButton *event)
     }
 
   if (GTK_WIDGET_CLASS(parent_class)->button_release_event != NULL)
-    return (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
-								   event);
-  return FALSE;
+    result = (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
+								     event);
+  else
+    result = FALSE;
+
+  if (check_expander)
+    {
+      if (gtk_tree_view_row_expanded(&view->view, path) == expanded)
+	{
+	  if (expanded)
+	    gtk_tree_view_collapse_row(&view->view, path);
+	  else
+	    gtk_tree_view_expand_row(&view->view, path, FALSE);
+	}
+    }
+
+  if (path != NULL)
+    gtk_tree_path_free(path);
+
+  return result;
 }
 
 
