@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.134 2004/01/24 07:04:14 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.139 2004/02/28 22:58:20 fuyu Exp $
  */
 
 #include "config.h"
@@ -558,7 +558,10 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board,
       g_signal_connect(G_OBJECT(board), "threadlist_write_thread_element",
 		       G_CALLBACK(threadlist_write_thread_element_cb),
 		       application);
-      info->recover_mode = application->recover_mode;
+      if (board->bbs_type == OCHUSHA_BBS_TYPE_2CH_HEADLINE)
+	info->recover_mode = FALSE;
+      else
+	info->recover_mode = application->recover_mode;
     }
 
   return info;
@@ -757,9 +760,25 @@ open_thread_view_cb(GtkWidget *selector, OchushaBBSThread *thread,
 {
   OchushaApplication *application = g_object_get_qdata(G_OBJECT(selector),
 						       application_id);
-  if (!with_browser && !with_browser)
-    paned_notebook_open_item_view(paned_notebook, thread, in_tab,
-				  application->select_tab_when_opened);
+  if (!with_browser && !fake_relation)
+    {
+      if (ochusha_bbs_thread_get_number_of_responses_on_server(thread)
+	  > ochusha_bbs_thread_get_number_of_responses_read(thread))
+	{
+	  GtkWidget *view = paned_notebook_get_item_view(paned_notebook,
+							 thread);
+	  GtkWidget *label = paned_notebook_get_tab_label(paned_notebook,
+							  thread);
+	  if (view != NULL && label != NULL)
+	    {
+	      GtkWidget *thread_view = gtk_bin_get_child(GTK_BIN(view));
+	      refresh_thread(application, thread_view, thread,
+			     ICON_LABEL(label));
+	    }
+	}
+      paned_notebook_open_item_view(paned_notebook, thread, in_tab,
+				    application->select_tab_when_opened);
+    }
   else
     {
       if (application != NULL)
@@ -777,11 +796,6 @@ open_thread_view_cb(GtkWidget *selector, OchushaBBSThread *thread,
 	      ochusha_open_url(application, url_buffer, in_tab, TRUE);
 	    }
 	}
-#if 0	/* Թͽ */
-      else
-	fprintf(stderr, "open_thread_view_cb: %s\n",
-		ochusha_bbs_thread_get_url_to_post_response(thread));
-#endif
     }
 }
 
@@ -1110,16 +1124,27 @@ open_thread_view(WorkerThread *employee, gpointer job_args)
 #endif
     /* λǡ٤Ϻ줿Ȥݾڤ */
 
+#if 1
+    if (thread == NULL && board->bbs_type == OCHUSHA_BBS_TYPE_2CH
+	&& application->config.login_2ch)
+      {
+	gchar title[256];
+	snprintf(title, 256, _("(tmp)%s"), args->thread_id);
+	thread = ochusha_bulletin_board_bbs_thread_new(board,
+						       args->thread_id,
+						       title);
+	thread->flags |= OCHUSHA_BBS_THREAD_DAT_DROPPED;
+	board->thread_list = g_slist_append(board->thread_list,
+					    thread);
+      }
+#endif
+
     if (thread != NULL)
       {
 	OCHU_THREADS_ENTER();
 
 	board_view = (PanedNotebook *)paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
 
-	/* TODO:
-	 * (board_view != NULL && thread == NULL)ʤ顢¿ʬΥDAT
-	 * 긵˥å夷ƤʬȤäƼɽǤ褦ˤ
-	 */
 	specify_thread_view_point(thread, board_view, args->res_num);
 
 	if (board_view != NULL)
@@ -1352,20 +1377,23 @@ open_kako_thread_view(WorkerThread *employee, gpointer job_args)
   {
     PanedNotebook *board_view;
     OchushaBBSThread *thread
-      = ochusha_bulletin_board_lookup_bbs_thread_by_url(board, args->url);
+      = ochusha_bulletin_board_lookup_bbs_thread_by_url(board,
+							application->broker,
+							args->url);
 
     /* ޤġġ*/
     while (info->is_busy && thread == NULL)
       {
 	THREAD_LIST_COND_WAIT;
 	thread = ochusha_bulletin_board_lookup_bbs_thread_by_url(board,
-								 args->url);
+							application->broker,
+							args->url);
       }
 
     if (thread == NULL)
       thread = ochusha_bulletin_board_lookup_kako_thread_by_url(board,
 							application->broker,
-							NULL, args->url);
+							args->url);
 
 #if DEBUG_GUI_MOST
     fprintf(stderr, "Kako Thread URL: \"%s\"\n", args->url);
@@ -1563,6 +1591,38 @@ start_rendering(gpointer data)
 }
 
 
+static void
+start_parsing_cb(EachThreadArgs *cb_args)
+{
+  OCHU_THREADS_ENTER();
+  threadlist_view_freeze(cb_args->view);
+}
+
+
+static void
+before_wait_cb(EachThreadArgs *cb_args)
+{
+  threadlist_view_chew(cb_args->view);
+  OCHU_THREADS_LEAVE();
+}
+
+
+static void
+after_wait_cb(EachThreadArgs *cb_args)
+{
+  OCHU_THREADS_ENTER();
+  threadlist_view_freeze(cb_args->view);
+}
+
+
+static void
+end_parsing_cb(EachThreadArgs *cb_args)
+{
+  threadlist_view_chew(cb_args->view);
+  OCHU_THREADS_LEAVE();
+}
+
+
 static void
 show_threadlist(WorkerThread *employee, gpointer args)
 {
@@ -1585,6 +1645,8 @@ show_threadlist(WorkerThread *employee, gpointer args)
       job_args->increment_ignored
     };
   ThreadlistRenderingSyncObject sync_object;
+  gboolean show_threadlist_non_incrementally
+    = application->show_threadlist_non_incrementally;
 
 #if DEBUG_GUI_MOST
   {
@@ -1648,6 +1710,9 @@ show_threadlist(WorkerThread *employee, gpointer args)
       goto finish_job;
     }
 
+  if (!show_threadlist_non_incrementally)
+    threadlist_view_show(view);
+
   if (application->sort_threadlist_by_weight)
     threadlist_view_set_sort_mode(view, THREADLIST_SORT_BY_WEIGHT);
   else
@@ -1703,10 +1768,10 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
   if (!ochusha_bulletin_board_refresh_threadlist(board, buffer,
 						 each_thread_cb,
-						 (StartParsingCallback *)OCHU_THREADS_ENTER_FUNC,
-						 (BeforeWaitCallback *)OCHU_THREADS_LEAVE_FUNC,
-						 (AfterWaitCallback *)OCHU_THREADS_ENTER_FUNC,
-						 (EndParsingCallback *)OCHU_THREADS_LEAVE_FUNC,
+						 (StartParsingCallback *)start_parsing_cb,
+						 (BeforeWaitCallback *)before_wait_cb,
+						 (AfterWaitCallback *)after_wait_cb,
+						 (EndParsingCallback *)end_parsing_cb,
 						 &cb_args))
     {
 #if DEBUG_GUI_MOST
@@ -1753,6 +1818,11 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
   OCHU_THREADS_ENTER();
 
+  if (show_threadlist_non_incrementally)
+    {
+      threadlist_view_show(view);
+    }
+
   /* ⤦ʤʤäƤɤ */
   g_object_unref(view);				/* бǧ */
   if (job_args->tab_label != NULL)
@@ -2068,8 +2138,13 @@ bookmark_thread(OchushaBulletinBoard *board, OchushaBBSThread *thread,
   const char *id = ochusha_bbs_thread_get_url(thread);
   OchushaBBSThread *proxy;
   gboolean bookmarked;
+  OchushaBulletinBoard *real_board;
 
   g_return_if_fail(IS_VIRTUAL_BOARD(board));
+  real_board = ochusha_bbs_thread_get_board(thread);
+  g_return_if_fail(OCHUSHA_IS_BULLETIN_BOARD(real_board));
+  if (real_board->bbs_type == OCHUSHA_BBS_TYPE_2CH_HEADLINE)
+    return;
 
   OCHU_THREADS_LEAVE();
   ochusha_bulletin_board_lock_thread_list(board);
@@ -2334,10 +2409,6 @@ item_view_required_cb(PanedNotebook *paned_notebook,
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*item_view),
 				     GTK_POLICY_AUTOMATIC,
 				     GTK_POLICY_AUTOMATIC);
-#if 0	/* themeǤˤޤ٤ */
-      gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(*item_view),
-					  GTK_SHADOW_ETCHED_IN);
-#endif
       gtk_container_add(GTK_CONTAINER(*item_view), thread_view);
       gtk_widget_show(*item_view);
     }
@@ -2656,7 +2727,8 @@ background_write_response(WorkerThread *employee, gpointer job_args)
 
   result = ochusha_bbs_thread_post_response(args->thread,
 					    args->application->broker,
-					    &response);
+					    &response,
+					    args->application->dolib_use_id_for_posting);
 
   if (result)
     {
