@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.71 2004/01/27 11:57:24 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.74 2004/02/28 23:45:13 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -99,6 +99,7 @@
 
 #define OCHUSHA_MINIMIZED_ICON			"Ochusha-Minimized"
 
+#define OCHUSHA_STOCK_MISSING_IMAGE		"Ochusha-Missing-Image"
 #define OCHUSHA_STOCK_WRITE_RESPONSE		"Ochusha-Write-Response"
 #define OCHUSHA_STOCK_NET_PROGRESS0		"Ochusha-Net-Progress-0"
 #define OCHUSHA_STOCK_NET_PROGRESS1		"Ochusha-Net-Progress-1"
@@ -224,6 +225,7 @@ struct _OchushaApplication
   char *threadlist_view_contents;
   gboolean use_builtin_calculate_weight;
   gboolean sort_threadlist_by_weight;
+  gboolean show_threadlist_non_incrementally;
 
   gboolean default_open_in_tab;
   gboolean select_tab_when_opened;
@@ -318,6 +320,14 @@ struct _OchushaApplication
 
   FILE *log_file;
 
+
+  /* DOLIBϢ */
+  char *dolib_id;
+  char *dolib_passwd;
+  gboolean dolib_save_passwd;
+  gboolean dolib_login_at_startup;
+  gboolean dolib_use_id_for_posting;
+
   OchushaConfig config;
 };
 
@@ -356,7 +366,6 @@ void ochusha_clipboard_set_text(OchushaApplication *application,
 
 void ochusha_open_url(OchushaApplication *application, const char *url,
 		      gboolean in_tab, gboolean use_web_browser);
-void ochusha_download_url(OchushaApplication *application, const char *url);
 #if ENABLE_MAIN_TOOLBAR
 void set_current_url(OchushaApplication *application, const char *url);
 const char *get_current_url(OchushaApplication *application);
