@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: regex_utils.c,v 1.5 2004/01/13 14:54:46 fuyu Exp $
+ * $Id: regex_utils.c,v 1.6 2004/01/14 10:03:18 fuyu Exp $
  */
 
 #include "config.h"
@@ -139,7 +139,7 @@ regex_editor_finalize(GObject *object)
     }
   if (editor->regex_buffer != NULL)
     {
-      OCHU_OBJECT_UNREF(G_OBJECT(editor->regex_buffer));
+      OCHU_OBJECT_UNREF(editor->regex_buffer);
       editor->regex_buffer = NULL;
     }
 
