@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: response_editor.c,v 1.10 2004/01/15 02:18:05 fuyu Exp $
+ * $Id: response_editor.c,v 1.11 2004/02/28 22:59:25 fuyu Exp $
  */
 
 #include "config.h"
@@ -38,6 +38,7 @@
 
 #include "utils.h"
 
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -81,7 +82,14 @@ response_editor_get_type(void)
 }
 
 
-GtkVBoxClass *parent_class;
+enum {
+  NOP_SIGNAL,
+  LAST_SIGNAL
+};
+
+
+static GtkVBoxClass *parent_class;
+static int response_editor_signals[LAST_SIGNAL] = { 0 };
 
 
 static void
@@ -89,6 +97,9 @@ response_editor_class_init(ResponseEditorClass *klass)
 {
   GObjectClass *o_class = (GObjectClass *)klass;
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
+#if 0
+  GtkBindingSet *binding_set = gtk_binding_set_by_class(klass);
+#endif
 
   parent_class = g_type_class_peek_parent(klass);
 
@@ -97,6 +108,20 @@ response_editor_class_init(ResponseEditorClass *klass)
 
   /* GtkObject signals */
   object_class->destroy = response_editor_destroy;
+
+  response_editor_signals[NOP_SIGNAL] =
+    g_signal_new("nop",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(ResponseEditorClass, nop),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+
+#if 0
+  gtk_binding_entry_add_signal(binding_set, GDK_Escape, 0,
+			       "nop", 0);
+#endif
 }
 
 
@@ -139,10 +164,6 @@ response_editor_init(ResponseEditor *editor)
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if 0	/* themeǤˤޤ٤ */
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
-				      GTK_SHADOW_ETCHED_IN);
-#endif
   gtk_widget_show(scrolled_window);
   gtk_box_pack_start(GTK_BOX(&editor->container),
 		     scrolled_window, TRUE, TRUE, 0);
