@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha.h,v 1.15 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha.h,v 1.16 2004/01/30 22:55:47 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_H_
@@ -47,13 +47,20 @@ extern "C" {
 #define OCHUSHA_USER_AGENT		"Monazilla/1.00 (Ochusha/" VERSION ")"
 
 /*
- * ʲĤ2ch˰¸ƤΤǡ̤ξΤ˾
+ * ʲ2ch˰¸ƤΤǡ̤ξΤ˾
  * 櫡ġ
  */
 #define OCHUSHA_DEFAULT_BBSMENU_URL	"http://www.ff.iij4u.or.jp/~ch2/bbsmenu.html"
 #define OCHUSHA_SUBJECT_TXT		"subject.txt"
 #define OCHUSHA_SUBBACK_HTML		"subback.html"
 
+#define OCHUSHA_DOLIB_HOST		"tiger2.he.net"
+#define OCHUSHA_DOLIB_URL		"https://tiger2.he.net/~tora3n2c/futen.cgi"
+#define OCHUSHA_DOLIB_USER_AGENT	"DOLIB/1.00"
+#define OCHUSHA_DOLIB_X_2CH_UA		"Ochusha/" VERSION
+#define OCHUSHA_CA_BUNDLE_CRT		PKGDATADIR "/ca-bundle.crt"
+
+
 /*
  * 2ch1000쥹¤ΤǡѤʾ1024
  * ʬJBBSФȤ⤽¤ʤäꤹ͡
@@ -77,6 +84,8 @@ struct _OchushaConfig
 
   /* İμѤURL: 2ch¸ʤΤ;ɤ٤⤷ */
   char *bbsmenu_url;
+  gboolean login_2ch;
+  char *session_id_2ch;
 
 
   /* proxyϢ */
@@ -94,7 +103,7 @@ struct _OchushaConfig
   int thread_chunksize;
 
 
-  /* network˥åȤ̤ */
+  /* network˥åȤ */
   gboolean offline;
 };
 
