@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.31 2004/01/27 17:44:53 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.33 2004/02/27 22:15:38 fuyu Exp $
  */
 
 #include "config.h"
@@ -165,7 +165,10 @@ static GSList *active_buffer_list = NULL;
 #define LOCK_BUFFER_LOCK(buffer)					\
   do									\
     {									\
-      int result = pthread_mutex_trylock(&buffer->sync_object->mutex);	\
+      int result;							\
+      if (buffer->sync_object == NULL)					\
+	break;								\
+      result = pthread_mutex_trylock(&buffer->sync_object->mutex);	\
       if (result != 0)							\
 	{								\
 	  if (result == EBUSY)						\
@@ -189,43 +192,55 @@ static GSList *active_buffer_list = NULL;
 	    }								\
 	}								\
     } while (0)
-#if 0
-#define LOCK_BUFFER_LOCK(buffer)				\
-  if (pthread_mutex_lock(&buffer->sync_object->mutex) != 0)	\
-    {								\
-      fprintf(stderr, "Couldn't lock a mutex.\n");		\
-      abort();							\
-    }
-#endif
 
-#define UNLOCK_BUFFER_LOCK(buffer)				\
-  if (pthread_mutex_unlock(&buffer->sync_object->mutex) != 0)	\
-    {								\
-      fprintf(stderr, "Couldn't unlock a mutex.\n");		\
-      abort();							\
-    }
+#define UNLOCK_BUFFER_LOCK(buffer)					\
+  do									\
+    {									\
+      if (buffer->sync_object == NULL)					\
+	break;								\
+      if (pthread_mutex_unlock(&buffer->sync_object->mutex) != 0)	\
+	{								\
+	  fprintf(stderr, "Couldn't unlock a mutex.\n");		\
+	  abort();							\
+	}								\
+    } while (0)
 
 #define WAIT_BUFFER_CONDITION(buffer)				\
-  if (pthread_cond_wait(&buffer->sync_object->cond,		\
-			&buffer->sync_object->mutex) != 0)	\
+  do								\
     {								\
-      fprintf(stderr, "Couldn't wait a condition.\n");		\
-      abort();							\
-    }
+      if (buffer->sync_object == NULL)				\
+	break;							\
+      if (pthread_cond_wait(&buffer->sync_object->cond,		\
+			    &buffer->sync_object->mutex) != 0)	\
+	{							\
+	  fprintf(stderr, "Couldn't wait a condition.\n");	\
+	  abort();						\
+	}							\
+    } while (0)
 
 #define SIGNAL_BUFFER_CONDITION(buffer)				\
-  if (pthread_cond_signal(&buffer->sync_object->cond) != 0)	\
+  do								\
     {								\
-      fprintf(stderr, "Couldn't siganl a condition.\n");	\
-      abort();							\
-    }
+      if (buffer->sync_object == NULL)				\
+	break;							\
+      if (pthread_cond_signal(&buffer->sync_object->cond) != 0)	\
+	{							\
+	  fprintf(stderr, "Couldn't siganl a condition.\n");	\
+	  abort();						\
+	}							\
+    } while (0)
 
-#define BROADCAST_BUFFER_CONDITION(buffer)			\
-  if (pthread_cond_broadcast(&buffer->sync_object->cond) != 0)	\
-    {								\
-      fprintf(stderr, "Couldn't siganl a condition.\n");	\
-      abort();							\
-    }
+#define BROADCAST_BUFFER_CONDITION(buffer)				\
+  do									\
+    {									\
+      if (buffer->sync_object == NULL)					\
+	break;								\
+      if (pthread_cond_broadcast(&buffer->sync_object->cond) != 0)	\
+	{								\
+	  fprintf(stderr, "Couldn't siganl a condition.\n");		\
+	  abort();							\
+	}								\
+    } while (0)
 
 
 enum {
@@ -243,7 +258,7 @@ enum {
 static int async_buffer_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
 
 
-void
+static void
 blind_awake(OchushaAsyncBuffer *buffer)
 {
   LOCK_BUFFER_LOCK(buffer);
@@ -628,6 +643,7 @@ ochusha_async_buffer_reset(OchushaAsyncBuffer *buffer)
   buffer->fixed = FALSE;
   buffer->state = OCHUSHA_ASYNC_BUFFER_OK;
   buffer->number_of_suspended_users = 0;
+  buffer->length = 0;
 
   UNLOCK_BUFFER_LOCK(buffer);
   return TRUE;
