@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.27 2004/01/18 04:50:26 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.29 2004/02/27 22:16:20 fuyu Exp $
  */
 
 #include "config.h"
@@ -588,7 +588,8 @@ ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread)
 gboolean
 ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
 				 OchushaNetworkBroker *broker,
-				 const OchushaBBSResponse *response)
+				 const OchushaBBSResponse *response,
+				 gboolean use_id)
 {
   OchushaBBSThreadClass *klass;
 
@@ -600,7 +601,7 @@ ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
 
   g_return_val_if_fail(klass->post_response != NULL, FALSE);
 
-  return (*klass->post_response)(thread, broker, response);
+  return (*klass->post_response)(thread, broker, response, use_id);
 }
 
 
@@ -673,7 +674,8 @@ ochusha_bbs_thread_remove_cache(OchushaBBSThread *thread,
   g_return_if_fail(klass->remove_cache != NULL);
 
   thread->number_of_responses_read = 0;
-  thread->flags &= ~OCHUSHA_BBS_THREAD_STOPPED;
+  thread->flags
+    &= ~(OCHUSHA_BBS_THREAD_STOPPED | OCHUSHA_BBS_THREAD_DAT_OYSTER);
 
   return (*klass->remove_cache)(thread, config);
 }
