@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.h,v 1.22 2004/01/18 04:50:26 fuyu Exp $
+ * $Id: ochusha_bbs_thread.h,v 1.24 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_THREAD_H_
@@ -134,7 +134,8 @@ struct _OchushaBBSThreadClass
   gboolean (*post_supported)(OchushaBBSThread *thread);
   gboolean (*post_response)(OchushaBBSThread *thread,
 			    OchushaNetworkBroker *broker,
-			    const OchushaBBSResponse *response);
+			    const OchushaBBSResponse *response,
+			    gboolean use_id);
 };
 
 
@@ -143,6 +144,7 @@ typedef enum
   OCHUSHA_BBS_THREAD_DAT_DROPPED = 1 << 0,
   OCHUSHA_BBS_THREAD_STOPPED = 1 << 1,
   OCHUSHA_BBS_THREAD_KAKO = 1 << 2,
+  OCHUSHA_BBS_THREAD_DAT_OYSTER = 1 << 3,
 } OchushaBBSThreadState;
 
 
@@ -218,7 +220,8 @@ gboolean ochusha_bbs_thread_preview_response(OchushaBBSThread *thread,
 
 gboolean ochusha_bbs_thread_post_response(OchushaBBSThread *thread,
 					  OchushaNetworkBroker *broker,
-					  const OchushaBBSResponse *response);
+					  const OchushaBBSResponse *response,
+					  gboolean use_id);
 
 #ifdef __cplusplus
 } // extern "C"
