@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.39 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.42 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #include "config.h"
@@ -52,7 +52,7 @@
 #include <zlib.h>
 
 
-#define ANALYSIS_INTERVAL	50
+#define ANALYSIS_INTERVAL	200
 
 
 static void ochusha_board_2ch_class_init(OchushaBoard2chClass *klass);
@@ -81,11 +81,11 @@ static OchushaBBSThread *ochusha_board_2ch_thread_new(
 						const gchar *title);
 static OchushaBBSThread *ochusha_board_2ch_lookup_thread_by_url(
 						OchushaBulletinBoard *board,
+						OchushaNetworkBroker *broker,
 						const char *url);
 static OchushaBBSThread *ochusha_board_2ch_lookup_kako_thread_by_url(
 						OchushaBulletinBoard *board,
 						OchushaNetworkBroker *broker,
-						OchushaAsyncBuffer *buffer,
 						const char *url);
 static OchushaAsyncBuffer *ochusha_board_2ch_get_threadlist_source(
 					OchushaBulletinBoard *board,
@@ -112,7 +112,8 @@ static gboolean ochusha_board_2ch_create_new_thread(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
 					const gchar *title,
-					const OchushaBBSResponse *response);
+					const OchushaBBSResponse *response,
+					gboolean use_id);
 
 GType
 ochusha_board_2ch_get_type(void)
@@ -283,6 +284,7 @@ ochusha_board_2ch_get_subject_txt_encoding(OchushaBulletinBoard *board)
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
       return "CP932";
 
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
@@ -427,6 +429,7 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
       {
 	char *tmp_pos = strchr(abs_path + 1, '/');
@@ -465,6 +468,7 @@ ochusha_board_2ch_thread_new(OchushaBulletinBoard *board, const char *id,
 
 static OchushaBBSThread *
 ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
+				       OchushaNetworkBroker *broker,
 				       const char *url)
 {
   OchushaBBSThread *thread = NULL;
@@ -490,9 +494,8 @@ ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
 #endif
 	    {
 	      if (g_ascii_strcasecmp(board->base_url, board_url) == 0)
-		thread
-		  = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
-								   thread_id);
+		thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
+									thread_id);
 	    }
 	}
     }
@@ -503,6 +506,11 @@ ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
   if (thread_id != NULL)
     G_FREE(thread_id);
 
+#if 1
+  if (thread == NULL)
+    fprintf(stderr, "ochusha_board_2ch_lookup_thread_by_url: %s\n", url);
+#endif
+
   return thread;
 }
 
@@ -510,7 +518,6 @@ ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
 static OchushaBBSThread *
 ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 					    OchushaNetworkBroker *broker,
-					    OchushaAsyncBuffer *buffer,
 					    const char *url)
 {
   OchushaBBSThread *thread = NULL;
@@ -534,7 +541,7 @@ ochusha_board_2ch_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 	  thread_2ch = OCHUSHA_THREAD_2CH(thread);
 	  ochusha_thread_2ch_set_kako_html_url(thread_2ch, url);
 	  dat_buffer = ochusha_bbs_thread_get_responses_source(thread, broker,
-					buffer,
+					NULL,
 					OCHUSHA_NETWORK_BROKER_CACHE_AS_IS);
 	  if (dat_buffer != NULL)
 	    {
@@ -578,6 +585,7 @@ ochusha_board_2ch_get_response_character_encoding(OchushaBulletinBoard *board)
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
       return "CP932";
 
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
@@ -598,6 +606,7 @@ ochusha_board_2ch_get_response_iconv_helper(OchushaBulletinBoard *board)
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
       return cp932_to_utf8_helper;
 
     default:
@@ -614,9 +623,10 @@ ochusha_board_2ch_is_new_thread_supported(OchushaBulletinBoard *board)
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
       return TRUE;
 
+    case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
     default:
       return FALSE;
     }
@@ -627,7 +637,8 @@ static gboolean
 ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
 				    OchushaNetworkBroker *broker,
 				    const gchar *title_utf8,
-				    const OchushaBBSResponse *response)
+				    const OchushaBBSResponse *response,
+				    gboolean use_id)
 {
   OchushaBoard2ch *board_2ch;
   char *tmp_string = NULL;
@@ -656,6 +667,7 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
       break;
 
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
     default:
       return FALSE;	/* ̤ݡ */
     }
@@ -715,6 +727,12 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
+      if (use_id && broker->config->login_2ch)
+	query = g_strdup_printf("submit=%%90%%56%%8B%%4B%%83%%58%%83%%8C%%83%%62%%83%%68%%8D%%EC%%90%%AC&subject=%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&sid=%s&time=%ld", title, from, mail, message, bbs, broker->config->session_id_2ch, time);
+      else
+	query = g_strdup_printf("submit=%%90%%56%%8B%%4B%%83%%58%%83%%8C%%83%%62%%83%%68%%8D%%EC%%90%%AC&subject=%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&time=%ld", title, from, mail, message, bbs, time);
+      break;
+
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
       query = g_strdup_printf("submit=%%90%%56%%8B%%4B%%83%%58%%83%%8C%%83%%62%%83%%68%%8D%%EC%%90%%AC&subject=%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&time=%ld", title, from, mail, message, bbs, time);
       break;
@@ -814,6 +832,7 @@ ochusha_board_2ch_get_read_cgi_url(OchushaBoard2ch *board_2ch)
 	{
 	case OCHUSHA_BBS_TYPE_2CH:
 	case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+	case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
 	case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
 	  if (snprintf(url, PATH_MAX, "http://%s/test/read.cgi",
 		       ochusha_bulletin_board_get_server(board)) < PATH_MAX)
@@ -988,6 +1007,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
       converter_helper = cp932_to_utf8_helper;
       break;
 
