@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.33 2004/01/24 07:02:35 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.35 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #include "config.h"
@@ -636,6 +636,7 @@ ochusha_bulletin_board_lookup_bbs_thread_by_id(OchushaBulletinBoard *board,
 
 OchushaBBSThread *
 ochusha_bulletin_board_lookup_bbs_thread_by_url(OchushaBulletinBoard *board,
+						OchushaNetworkBroker *broker,
 						const char *url)
 {
   OchushaBulletinBoardClass *klass;
@@ -646,14 +647,13 @@ ochusha_bulletin_board_lookup_bbs_thread_by_url(OchushaBulletinBoard *board,
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->lookup_thread_by_url != NULL, NULL);
 
-  return (*klass->lookup_thread_by_url)(board, url);
+  return (*klass->lookup_thread_by_url)(board, broker, url);
 }
 
 
 OchushaBBSThread *
 ochusha_bulletin_board_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
 						 OchushaNetworkBroker *broker,
-						 OchushaAsyncBuffer *buffer,
 						 const char *url)
 {
   OchushaBulletinBoardClass *klass;
@@ -664,7 +664,7 @@ ochusha_bulletin_board_lookup_kako_thread_by_url(OchushaBulletinBoard *board,
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->lookup_kako_thread_by_url != NULL, NULL);
 
-  return (*klass->lookup_kako_thread_by_url)(board, broker, buffer, url);
+  return (*klass->lookup_kako_thread_by_url)(board, broker, url);
 }
 
 
@@ -797,7 +797,8 @@ gboolean
 ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
 					 OchushaNetworkBroker *broker,
 					 const gchar *title,
-					 const OchushaBBSResponse *response)
+					 const OchushaBBSResponse *response,
+					 gboolean use_id)
 {
   OchushaBulletinBoardClass *klass;
 
@@ -810,7 +811,7 @@ ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
 
   g_return_val_if_fail(klass->create_new_thread != NULL, FALSE);
 
-  return (*klass->create_new_thread)(board, broker, title, response);
+  return (*klass->create_new_thread)(board, broker, title, response, use_id);
 }
 
 
