@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.22 2004/01/24 07:02:35 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.25 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -67,6 +67,7 @@ enum
   OCHUSHA_BBS_TYPE_JBBS_SHITARABA,
   OCHUSHA_BBS_TYPE_MITINOKU,
   OCHUSHA_BBS_TYPE_2CH_COMPATIBLE,
+  OCHUSHA_BBS_TYPE_2CH_HEADLINE,
 };
 
 
@@ -128,10 +129,10 @@ struct _OchushaBulletinBoardClass
   OchushaBBSThread *(*thread_new)(OchushaBulletinBoard *board,
 				  const char *id, const gchar *title);
   OchushaBBSThread *(*lookup_thread_by_url)(OchushaBulletinBoard *board,
+					    OchushaNetworkBroker *broker,
 					    const char *url);
   OchushaBBSThread *(*lookup_kako_thread_by_url)(OchushaBulletinBoard *board,
 						 OchushaNetworkBroker *broker,
-						 OchushaAsyncBuffer *buffer,
 						 const char *url);
   OchushaAsyncBuffer *(*get_threadlist_source)(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
@@ -160,7 +161,8 @@ struct _OchushaBulletinBoardClass
   gboolean (*create_new_thread)(OchushaBulletinBoard *board,
 				OchushaNetworkBroker *broker,
 				const gchar *title,
-				const OchushaBBSResponse *response);
+				const OchushaBBSResponse *response,
+				gboolean use_id);
 };
 
 
@@ -196,11 +198,11 @@ OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_id(
 					const char *id);
 OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_url(
 					OchushaBulletinBoard *board,
+					OchushaNetworkBroker *broker,
 					const char *url);
 OchushaBBSThread *ochusha_bulletin_board_lookup_kako_thread_by_url(
 					OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
-					OchushaAsyncBuffer *buffer,
 					const char *url);
 
 OchushaAsyncBuffer *ochusha_bulletin_board_get_threadlist_source(
@@ -236,7 +238,8 @@ gboolean ochusha_bulletin_board_is_new_thread_supported(
 gboolean ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
 					const gchar *title,
-					const OchushaBBSResponse *response);
+					const OchushaBBSResponse *response,
+					gboolean use_id);
 
 gboolean ochusha_bulletin_board_read_threadlist_xml(
 					OchushaBulletinBoard *board,
