@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.53 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.60 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #include "config.h"
@@ -32,7 +32,6 @@
 #include "ochusha.h"
 #include "ochusha_network_broker.h"
 
-#include "gziputils.h"
 #include "worker.h"
 
 #include "marshal.h"
@@ -55,7 +54,6 @@
 
 
 #define DISABLE_PERSISTENT_CONNECTION		0
-#define ENABLE_GZIPPED_TRANSFER			1
 #define ENABLE_GZIPPED_DIFFERENCIAL_READ	0
 
 
@@ -63,8 +61,8 @@
 #define HEADER_BUFFER_LENGTH	256
 #define LOG_MESSAGE_SIZE	4096
 
-#define POLLING_INTERVAL_MILLIS	100
-#define POLLING_TIMEOUT_COUNT	300
+#define POLLING_INTERVAL_MILLIS	500
+#define POLLING_TIMEOUT_COUNT	60
 
 
 static void ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass);
@@ -341,13 +339,14 @@ write_buffer_to_cache(OchushaNetworkBroker *broker, const char *url,
 static gboolean
 setup_common_request_headers(OchushaNetworkBroker *broker,
 			     ghttp_request *request,
-			     gboolean posting)
+			     gboolean posting, gboolean allow_proxy)
 {
   gchar message[LOG_MESSAGE_SIZE];
   gboolean result = TRUE;
   const OchushaConfig *config = broker->config;
 
-  if ((config->enable_proxy
+  if (allow_proxy &&
+      (config->enable_proxy
        || (config->enable_proxy_only_for_posting && posting))
       && config->proxy_url != NULL)
     {
@@ -391,38 +390,6 @@ setup_common_request_headers(OchushaNetworkBroker *broker,
 }
 
 
-typedef enum
-{
-  UNKNOWN_ENCODING = -1,
-  IDENTITY_ENCODING,
-  GZIP_ENCODING,
-  DEFLATE_ENCODING,
-} ContentEncodingType;
-
-
-static ContentEncodingType
-get_content_encoding_type(ghttp_request *request)
-{
-  const char *encoding = ghttp_get_header(request, http_hdr_Content_Encoding);
-
-  if (encoding != NULL)
-    {
-      if (strcmp(encoding, "gzip") == 0)
-	return GZIP_ENCODING;
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "Content-Encoding: %s\n", encoding);
-#endif
-      if (strcmp(encoding, "deflate") == 0)
-	return DEFLATE_ENCODING;
-      if (strcmp(encoding, "identity") == 0)
-	return IDENTITY_ENCODING;
-      return UNKNOWN_ENCODING;
-    }
-
-  return IDENTITY_ENCODING;
-}
-
-
 /*
  * OchushaAsyncBufferbroker_job_args_idqdataȤƤäĤ롣
  */
@@ -431,6 +398,7 @@ typedef struct _NetworkBrokerJobArgs
   int length;
   int chunksize;
   char *url;
+  char *cache_url;
   char *if_modified_since;
 
   ghttp_request *request;
@@ -780,34 +748,24 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   int sock_fd = -1;
   int status_code = 0;
   ghttp_request *request = NULL;
-  ContentEncodingType content_encoding = UNKNOWN_ENCODING;
-  GzipBuffer *gzip_buffer = NULL;
   ghttp_status state = ghttp_not_done;
   ghttp_current_status current_status;
 
-  const char *tmp_pos;
-  gboolean force_gunzip;	/* Ķad-hock塼fordat.gz */
-
   ochusha_async_buffer_reset(buffer);
   ochusha_async_buffer_update_length(buffer, 0);
 
-  if ((tmp_pos = strstr(url, ".dat.gz")) != NULL && tmp_pos[7] == '\0')
-    force_gunzip = TRUE;
-  else
-    force_gunzip = FALSE;
-
   request = ghttp_request_new();
-#if 1
+
   if (args->chunksize > 0)
     ghttp_set_chunksize(request, args->chunksize);
-#endif
+
   args->request = request;	/* 塹Τˡġ */
 
-  ghttp_set_uri(request, (char *)url);
+  ghttp_set_uri(request, url);
   ghttp_set_type(request, ghttp_type_get);
-#if ENABLE_GZIPPED_TRANSFER
-  ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
-#endif
+
+  if (g_str_has_suffix(url, ".dat.gz") || !g_str_has_suffix(url, ".gz"))
+    ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
 
 #if DISABLE_PERSISTENT_CONNECTION
   ghttp_set_header(request, http_hdr_Connection, "close");
@@ -824,7 +782,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   else
     if_modified_since = NULL;
 
-  if (!setup_common_request_headers(broker, request, FALSE))
+  if (!setup_common_request_headers(broker, request, FALSE, TRUE))
     {
       ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
@@ -854,7 +812,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   while (TRUE)
     {
       int body_len;
-      char *body;
+      const char *body;
       ghttp_proc prev_proc;
       gboolean read_ok;
 
@@ -870,9 +828,16 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else if ((read_ok = sync_object->read_ok)
 	       || (state == ghttp_not_done && sock_fd == -1))
 	{
+	  sync_object->timeout_count = 0;
 	  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 	  state = ghttp_process(request);
 	  current_status = ghttp_get_status(request);
+	  if (state == ghttp_not_done && prev_proc == ghttp_proc_response_hdrs
+	      && current_status.proc == ghttp_proc_response)
+	    {
+	      state = ghttp_process(request);
+	      current_status = ghttp_get_status(request);
+	    }
 	}
       else
 	{
@@ -912,6 +877,8 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	}
       else
 	{
+	  fprintf(stderr, "state = %d, read_ok = %d, sock_fd = %d\n",
+		  state, read_ok, sock_fd);
 	  fprintf(stderr, "prev_proc = %d\n", prev_proc);
 	  fprintf(stderr, "current_status.proc = %d\n", current_status.proc);
 	}
@@ -953,19 +920,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
       if (state == ghttp_not_done)
 	{
-	  const char *tmp_header;
-	  if (prev_proc == ghttp_proc_response_hdrs
-	      && (tmp_header = ghttp_get_header(request,
-						http_hdr_Transfer_Encoding))
-	      && strcasecmp(tmp_header, "chunked") == 0)
-	    {
-	      /* ɤlibghttpasync⡼ɤǤ
-	       * Transfer-Encoding: chunkedפϡޤȤ˼갷ʤ͡
-	       */
-	      ghttp_set_sync(request, ghttp_sync);
-	      read_ok = FALSE;
-	    }
-
 	  if (read_ok)
 	    ghttp_flush_response_buffer(request);
 	}
@@ -996,101 +950,24 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 				  current_status.bytes_read,
 				  current_status.bytes_total);
 
-	  if (content_encoding == UNKNOWN_ENCODING)
+	  if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 	    {
-	      /* ˤ1٤ʤ */
-	      content_encoding = get_content_encoding_type(request);
+	      ghttp_request_destroy(request);
+	      args->request = NULL;
 
-	      switch (content_encoding)
-		{
-		case GZIP_ENCODING:
-		  gzip_buffer = gzip_buffer_new(buffer);
-		  if (gzip_buffer == NULL)
-		    {
-		      error_message = _("Out of memory.");
-		      goto error_exit;
-		    }
-		  break;
+	      status->state
+		= OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
 
-		case DEFLATE_ENCODING:
-		  gzip_buffer = gzip_buffer_new(buffer);
-		  if (gzip_buffer == NULL)
-		    {
-		      error_message = _("Out of memory.");
-		      goto error_exit;
-		    }
-		  gzip_buffer_skip_gzip_header(gzip_buffer);
-		  break;
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Access terminated: %s\n"),
+		       url);
+	      ochusha_network_broker_output_log(broker, message);
 
-		case UNKNOWN_ENCODING:
-		  error_message = _("Unknown transfer encoding.");
-		  goto error_exit;
+	      LOCK_WORKER_SYNC_OBJECT(sync_object);
+	      sync_object->finished = TRUE;
+	      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 
-		case IDENTITY_ENCODING:
-		  if (force_gunzip)
-		    {
-		      gzip_buffer = gzip_buffer_new(buffer);
-		      if (gzip_buffer == NULL)
-			{
-			  error_message = _("Out of memory.");
-			  goto error_exit;
-			}
-		      content_encoding = GZIP_ENCODING;
-		    }
-		  break;
-		}
-	    }
-
-	  if (content_encoding == GZIP_ENCODING
-	      || content_encoding == DEFLATE_ENCODING)
-	    {
-	      GzipBufferStatus result
-		= gzip_buffer_append_data(gzip_buffer, body, body_len);
-	      if (result == GZIP_BUFFER_ERROR)
-		{
-		  error_message = _("Inflation failed.");
-		  gzip_buffer_free(gzip_buffer);
-
-		  goto error_exit;
-		}
-	      if (result == GZIP_BUFFER_INFLATION_DONE)
-		{
-		  gzip_buffer_free(gzip_buffer);
-		  gzip_buffer = NULL;
-		  if (state != ghttp_done)
-		    {
-		      snprintf(message, LOG_MESSAGE_SIZE,
-			       _("Requested file seems to contain garbage: %s\n"),
-			       url);
-		      ochusha_network_broker_output_log(broker, message);
-#if DEBUG_NETWORK_MOST
-		      fprintf(stderr, "Requested file seems to contain garbage: %s\n", url);
-#endif
-		    }
-		  break;
-		}
-	    }
-	  else
-	    {
-	      if (!ochusha_async_buffer_append_data(buffer, body, body_len))
-		{	/* content_encoding == IDENTITY_ENCODING */
-		  ghttp_request_destroy(request);
-		  args->request = NULL;
-
-		  status->state
-		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-
-		  snprintf(message, LOG_MESSAGE_SIZE,
-			   _("Access terminated: %s\n"),
-			   url);
-		  ochusha_network_broker_output_log(broker, message);
-
-		  LOCK_WORKER_SYNC_OBJECT(sync_object);
-		  sync_object->finished = TRUE;
-		  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
-
-		  return FALSE;
-		}
+	      return FALSE;
 	    }
 	}
       else
@@ -1121,7 +998,8 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
       LOCK_WORKER_SYNC_OBJECT(sync_object);
       sync_object->read_ok = FALSE;
-      sync_object->timeout_count = 0;
+      if (read_ok)
+	sync_object->timeout_count = 0;
 
       if (current_status.proc > ghttp_proc_request)
 	{
@@ -1139,14 +1017,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       UNLOCK_WORKER_SYNC_OBJECT(sync_object);
     }
 
-  if (gzip_buffer != NULL)
-    {
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "What's happen?  Connection was closed but inflation hasn't been completed.\n");
-#endif
-      gzip_buffer_free(gzip_buffer);
-    }
-
   if (status_code == 200)
     {
       const char *tmp_header
@@ -1272,6 +1142,9 @@ destruct_job_args(NetworkBrokerJobArgs *job_args)
   if (job_args->url != NULL)
     G_FREE(job_args->url);
 
+  if (job_args->cache_url != NULL)
+    G_FREE(job_args->cache_url);
+
   if (job_args->if_modified_since != NULL)
     G_FREE(job_args->if_modified_since);
 
@@ -1322,6 +1195,7 @@ static OchushaAsyncBuffer *
 ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
 					    OchushaAsyncBuffer *buffer,
 					    const char *url,
+					    const char *cache_url,
 					    const char *if_modified_since,
 					    JobFunc *job_function,
 					    gboolean modestly, int chunksize)
@@ -1344,6 +1218,11 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   g_signal_connect_object(G_OBJECT(buffer), "wakeup_now",
 			  G_CALLBACK(wakeup_now_cb), sync_object, 0);
 
+#if DEBUG_NETWORK_MOST
+  fprintf(stderr, "\"%s\" installing: timeout_cb(%p) count=%d\n",
+	  url, sync_object, G_OBJECT(sync_object)->ref_count);
+#endif
+
   sync_object->timeout_id
     = ochusha_network_broker_timeout_add_full(sync_object->broker,
 					      G_PRIORITY_DEFAULT - 1,
@@ -1357,6 +1236,7 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   args->length = buffer->length;
   buffer->length = 0;
   args->url = G_STRDUP(url);
+  args->cache_url = G_STRDUP(cache_url);
   args->chunksize = chunksize;
   if (if_modified_since != NULL
       && (args->length > 0
@@ -1397,7 +1277,6 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   char *url;
   int len;
   ghttp_request *request = NULL;
-  ContentEncodingType content_encoding = UNKNOWN_ENCODING;
   int status_code = 0;
   const char *error_message = NULL;
   gchar message[LOG_MESSAGE_SIZE];
@@ -1449,7 +1328,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   url = args->url;
 
   if (args->length == 0)
-    len = read_cache_to_buffer(broker->config, url, buffer);
+    len = read_cache_to_buffer(broker->config, args->cache_url, buffer);
   else
     {
       len = args->length;
@@ -1479,7 +1358,6 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       char header[HEADER_BUFFER_LENGTH];
       char cache_compare_buffer[CACHE_COMPARE_SIZE];
       int offset = len - CACHE_COMPARE_SIZE;
-      GzipBuffer *gzip_buffer = NULL;
       const char *tmp_header;
       ghttp_current_status current_status;
 
@@ -1495,10 +1373,9 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
       request = ghttp_request_new();
-#if 1
       if (args->chunksize > 0)
 	ghttp_set_chunksize(request, args->chunksize);
-#endif
+
       args->request = request;
       if (request == NULL)
 	{
@@ -1513,13 +1390,12 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       ghttp_set_type(request, ghttp_type_get);
       ghttp_set_header(request, http_hdr_Connection, "close");
 
-#if ENABLE_GZIPPED_TRANSFER
-# if ENABLE_GZIPPED_DIFFERENCIAL_READ
+#if ENABLE_GZIPPED_DIFFERENCIAL_READ
       ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
-# else
+#else
       ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate");
-# endif
 #endif
+
       ghttp_set_header(request, http_hdr_Range, header);
 
       if (args->if_modified_since != NULL)
@@ -1530,7 +1406,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  fprintf(stderr, "If-Modified-Since: %s\n", args->if_modified_since);
 #endif
 	}
-      if (!setup_common_request_headers(broker, request, FALSE))
+      if (!setup_common_request_headers(broker, request, FALSE, TRUE))
 	{
 	  /* ۤäƥåϤɤΤ */
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
@@ -1577,7 +1453,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       while (TRUE)
 	{
 	  int body_len;
-	  char *body;
+	  const char *body;
 	  gboolean before_cache_check = TRUE;
 	  ghttp_proc prev_proc;
 	  gboolean read_ok;
@@ -1595,10 +1471,17 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  else if ((read_ok = sync_object->read_ok)
 		   || (state == ghttp_not_done && sock_fd == -1))
 	    {
-	      sync_object->read_ok = FALSE;
+	      sync_object->timeout_count = 0;
 	      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 	      state = ghttp_process(request);
 	      current_status = ghttp_get_status(request);
+	      if (state == ghttp_not_done
+		  && prev_proc == ghttp_proc_response_hdrs
+		  && current_status.proc == ghttp_proc_response)
+		{
+		  state = ghttp_process(request);
+		  current_status = ghttp_get_status(request);
+		}
 	    }
 	  else
 	    {
@@ -1642,54 +1525,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		{
 		  goto cache_is_dirty;
 		}
-
-	      content_encoding = get_content_encoding_type(request);
-
-	      switch (content_encoding)
-		{
-		case GZIP_ENCODING:
-		  gzip_buffer = gzip_buffer_new(buffer);
-		  if (gzip_buffer == NULL)
-		    {
-		      error_message = _("Out of memory.");
-		      goto error_exit;
-		    }
-		  break;
-
-		case DEFLATE_ENCODING:
-		  gzip_buffer = gzip_buffer_new(buffer);
-		  if (gzip_buffer == NULL)
-		    {
-		      error_message = _("Out of memory.");
-		      goto error_exit;
-		    }
-		  gzip_buffer_skip_gzip_header(gzip_buffer);
-		  break;
-
-		case UNKNOWN_ENCODING:
-		  goto cache_is_dirty;	/* ĩ */
-
-		case IDENTITY_ENCODING:
-		  break;
-		}
 	    }
 
 	  if (state == ghttp_not_done)
 	    {
-	      const char *tmp_header;
-	      if (prev_proc == ghttp_proc_response_hdrs
-		  && (tmp_header = ghttp_get_header(request,
-						http_hdr_Transfer_Encoding))
-		  && strcasecmp(tmp_header, "chunked") == 0)
-		{
-		  /* ɤlibghttpasync⡼ɤǤ
-		   * Transfer-Encoding: chunkedפϡޤȤ˼갷ʤ
-		   * ͡
-		   */
-		  ghttp_set_sync(request, ghttp_sync);
-		  read_ok = FALSE;
-		}
-
 	      if (read_ok)
 		ghttp_flush_response_buffer(request);
 	    }
@@ -1717,36 +1556,13 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 				      current_status.bytes_read,
 				      current_status.bytes_total);
 
-	      if (content_encoding == GZIP_ENCODING
-		  || content_encoding == DEFLATE_ENCODING)
+	      if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 		{
-		  GzipBufferStatus result
-		    = gzip_buffer_append_data(gzip_buffer, body, body_len);
-		  if (result == GZIP_BUFFER_ERROR)
-		    {
-		      error_message = _("Inflation failed.");
-		      gzip_buffer_free(gzip_buffer);
-		      goto error_exit;
-		    }
-		  if (result == GZIP_BUFFER_INFLATION_DONE)
-		    {
-		      gzip_buffer_free(gzip_buffer);
-		      gzip_buffer = NULL;
-#if DEBUG_NETWORK_MOST
-		      if (state != ghttp_done)
-			fprintf(stderr, "Requested file contains garbage?\n");
-#endif
-		      break;
-		    }
-		}
-	      else if (!ochusha_async_buffer_append_data(buffer, body,
-							 body_len))
-		{	/* content_encoding == IDENTITY_ENCODING */
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
-
+		  
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"), url);
 		  ochusha_async_buffer_emit_access_failed(buffer,
@@ -1802,7 +1618,8 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  LOCK_WORKER_SYNC_OBJECT(sync_object);
 	  sync_object->read_ok = FALSE;
-	  sync_object->timeout_count = 0;
+	  if (read_ok)
+	    sync_object->timeout_count = 0;
 
 	  if (current_status.proc > ghttp_proc_request)
 	    {
@@ -1841,7 +1658,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       ghttp_request_destroy(request);
       args->request = NULL;
-      write_buffer_to_cache(broker, url, buffer);
+      write_buffer_to_cache(broker, args->cache_url, buffer);
 
       snprintf(message, LOG_MESSAGE_SIZE, _("Cache updated: %s\n"), url);
       ochusha_network_broker_output_log(broker, message);
@@ -1887,7 +1704,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS)
 	status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
 
-      write_buffer_to_cache(broker, url, buffer);
+      write_buffer_to_cache(broker, args->cache_url, buffer);
     }
   ochusha_async_buffer_fix(buffer);
   ochusha_async_buffer_active_unref(buffer);
@@ -2003,14 +1820,14 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       /* ̿ */
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
       /* ХåեƤ򥭥å˽񤭹ࡣ*/
-      write_buffer_to_cache(broker, args->url, buffer);
+      write_buffer_to_cache(broker, args->cache_url, buffer);
     }
   else
     {
       /* ̿Ԥå夬(if_modified_sinceNULL)ʾ */
       /* ochusha_async_buffer_reset(buffer); */
       ochusha_async_buffer_update_length(buffer, 0);
-      if (read_cache_to_buffer(broker->config, args->url, buffer))
+      if (read_cache_to_buffer(broker->config, args->cache_url, buffer))
 	{
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
 	  if (status->status_code == 304)
@@ -2196,9 +2013,58 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 				     const char *if_modified_since,
 				     OchushaNetworkBrokerCacheMode mode,
 				     gboolean modestly, int chunksize)
+{
+  return ochusha_network_broker_read_from_url_full(broker, buffer, url,
+						   NULL, if_modified_since,
+						   mode, modestly, chunksize);
+}
+
+
+/*
+ * ochusha_network_broker_read_from_url_full
+ *
+ * NULLcache_urlͿ줿硢cache_url򥭡Ȥƥåθ
+ * ¸Ԥcache_urlNULLǤϡɤ߹URL(url)򥭡Ȥ
+ * Ѥ롣
+ *
+ * Ϳ줿URLǡɤ߹ࡣ
+ * offline⡼ɤǥå夬¸ߤʤNULL֤
+ *
+ * modeOCHUSHA_NETWORK_BROKER_CACHE_IGNOREǤ硢ɬͥåȥ
+ * ǡɤ߹ߡ̤򥭥å夷ʤ
+ *
+ * ʳξˤɤ߹߻˲餫ηǥå夬ФȤ
+ * ˥ͥåȥɹǤˤϥǡ򥭥å¸롣
+ *
+ * OCHUSHA_NETWORK_BROKER_CACHE_AS_ISξ硢å夬¸ߤˤϤ
+ * Τޤ޻Ȥ
+ *
+ * OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATEξ硢å夵Ƥǡ
+ * append onlyǹΤǤȸʤå夵Ƥǡ
+ * ǿǤκʬΤߤͥåȥɤ߹褦ĩ魯롣⤷append only
+ * ǹǤʤäˤɤ߹ľ
+ *
+ * OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESHξ硢å夬ͥå
+ * 鴰˥ǡɤ߹ߡͥåȥɹԲǽǤä
+ * ˤΤߡå夵ƤǡȤ
+ *
+ * ͥåȥ˥硢̿˼ԤǤĹ0ΥХåե
+ * ֤Τա
+ */
+OchushaAsyncBuffer *
+ochusha_network_broker_read_from_url_full(OchushaNetworkBroker *broker,
+					  OchushaAsyncBuffer *buffer,
+					  const char *url,
+					  const char *cache_url,
+					  const char *if_modified_since,
+					  OchushaNetworkBrokerCacheMode mode,
+					  gboolean modestly, int chunksize)
 {
   g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
 		       && broker->config != NULL && url != NULL, NULL);
+
+  if (cache_url == NULL)
+    cache_url = url;
 #if DEBUG_NETWORK_MOST
   fprintf(stderr, "ochusha_network_broker_read_from_url: %s\n", url);
 #endif
@@ -2222,7 +2088,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       if (buffer != NULL && buffer->length > 0)
 	return buffer;
 
-      fd = ochusha_config_cache_open_file(broker->config, url, O_RDONLY);
+      fd = ochusha_config_cache_open_file(broker->config, cache_url, O_RDONLY);
       if (fd >= 0)
 	{
 	  WorkerJob *job = G_NEW0(WorkerJob, 1);
@@ -2280,20 +2146,21 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
     {
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATE:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
-						url, if_modified_since,
+						url, cache_url,
+						if_modified_since,
 						(JobFunc *)try_update_cache,
 						modestly, chunksize);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH:
     case OCHUSHA_NETWORK_BROKER_CACHE_AS_IS:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
-					url, if_modified_since,
+					url, cache_url, if_modified_since,
 					(JobFunc *)refresh_cache_after_read,
 					modestly, chunksize);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_IGNORE:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
-							 url, NULL,
+							 url, cache_url, NULL,
 							 (JobFunc *)force_read,
 							 modestly, chunksize);
 
@@ -2306,305 +2173,6 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 }
 
 
-static gboolean
-http_read_from_url_synchronously(OchushaNetworkBroker *broker,
-				 OchushaAsyncBuffer *buffer)
-{
-  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
-					      broker_job_args_id);
-  OchushaNetworkBrokerBufferStatus *status
-    = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
-  const char *url = args->url;
-  const char *error_message = NULL;
-  const char *body;
-  int body_len;
-  int status_code = 0;
-  ghttp_request *request = NULL;
-  ContentEncodingType content_encoding = UNKNOWN_ENCODING;
-  GzipBuffer *gzip_buffer = NULL;
-  ghttp_status state;
-
-  request = ghttp_request_new();
-#if 1
-  if (args->chunksize > 0)
-    ghttp_set_chunksize(request, args->chunksize);
-#endif
-  args->request = request;	/* 塹Τˡġ */
-
-  ghttp_set_uri(request, (char *)url);
-  ghttp_set_type(request, ghttp_type_get);
-#if ENABLE_GZIPPED_TRANSFER
-  ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
-#endif
-  ghttp_set_header(request, http_hdr_Connection, "close");
-
-  if (!setup_common_request_headers(broker, request, FALSE))
-    {
-      ochusha_async_buffer_emit_access_failed(buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
-			_("Proxy setting may be wrong."));
-
-      ghttp_request_destroy(request);
-      args->request = NULL;
-      return FALSE;
-    }    
-
-  ochusha_async_buffer_emit_access_started(buffer);
-
-  ghttp_set_sync(request, ghttp_sync);
-  ghttp_prepare(request);
-  state = ghttp_process(request);
-
-  if (state == ghttp_error)
-    {
-      error_message = _("Unknown I/O error.");
-      goto error_exit;
-    }
-
-  status_code = ghttp_status_code(request);
-  status->status_code = status_code;
-
-  if (status_code == 200)
-    {
-      const char *tmp_header
-	= ghttp_get_header(request, http_hdr_Last_Modified);
-      if (tmp_header != NULL)
-	status->last_modified = G_STRDUP(tmp_header);
-      else
-	status->last_modified = NULL;
-      tmp_header = ghttp_get_header(request, http_hdr_Date);
-      if (tmp_header != NULL)
-	status->date = G_STRDUP(tmp_header);
-      else
-	status->date = NULL;
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "Synchronous read done.\n");
-#endif
-    }
-  else
-    {
-      goto error_exit;
-    }
-
-  body = ghttp_get_body(request);
-  body_len = ghttp_get_body_len(request);
-      
-  if (body_len > 0)
-    {
-      ghttp_current_status current_status = ghttp_get_status(request);
-      if (current_status.bytes_total > 0)
-	ochusha_async_buffer_emit_access_progressed(buffer,
-			      current_status.bytes_read,
-			      current_status.bytes_total);
-
-      content_encoding = get_content_encoding_type(request);
-
-      switch (content_encoding)
-	{
-	case GZIP_ENCODING:
-	  gzip_buffer = gzip_buffer_new(buffer);
-	  if (gzip_buffer == NULL)
-	    {
-	      error_message = _("Out of memory.");
-	      goto error_exit;
-	    }
-	  break;
-
-	case DEFLATE_ENCODING:
-	  gzip_buffer = gzip_buffer_new(buffer);
-	  if (gzip_buffer == NULL)
-	    {
-	      error_message = _("Out of memory.");
-	      goto error_exit;
-	    }
-	  gzip_buffer_skip_gzip_header(gzip_buffer);
-	  break;
-
-	case UNKNOWN_ENCODING:
-	  error_message = _("Unknown transfer encoding.");
-	  goto error_exit;
-  
-	case IDENTITY_ENCODING:
-	  break;
-	}
-
-      if (content_encoding == GZIP_ENCODING
-	  || content_encoding == DEFLATE_ENCODING)
-	{
-	  GzipBufferStatus result
-	    = gzip_buffer_append_data(gzip_buffer, body, body_len);
-	  if (result == GZIP_BUFFER_ERROR)
-	    {
-	      error_message = _("Inflation failed.");
-	      gzip_buffer_free(gzip_buffer);
-	      goto error_exit;
-	    }
-	  if (result == GZIP_BUFFER_INFLATION_DONE)
-	    {
-	      gzip_buffer_free(gzip_buffer);
-	      gzip_buffer = NULL;
-#if DEBUG_NETWORK_MOST
-	      if (state != ghttp_done)
-		fprintf(stderr, "Requested file contains garbage?\n");
-#endif
-	    }
-	}
-      else if (!ochusha_async_buffer_append_data(buffer, body, body_len))
-	{	/* content_encoding == IDENTITY_ENCODING */
-	  ghttp_request_destroy(request);
-	  args->request = NULL;
-
-	  status->state
-	    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-
-	  return FALSE;
-	}
-    }
-
-  if (gzip_buffer != NULL)
-    {
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "What's happen?  Connection was closed but inflation hasn't been completed.\n");
-#endif
-      gzip_buffer_free(gzip_buffer);
-    }
-
-#if DEBUG_NETWORK_MOST
-  fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
-#endif
-  ochusha_async_buffer_emit_access_finished(buffer);
-
-  return TRUE;
-
- error_exit:
-#if DEBUG_NETWORK_MOST
-  fprintf(stderr, "force_read_synchronously(): error.\n");
-#endif
-  if (error_message != NULL)
-    {
-      ochusha_async_buffer_emit_access_failed(buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			error_message);
-#if 0
-      fprintf(stderr, "access failed: status_code=%d %s\n",
-	      status_code, error_message);
-#endif
-    }
-  else if (status_code != 0)
-    {
-      char message[LOG_MESSAGE_SIZE];
-      snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)\n",
-	       status_code, ghttp_reason_phrase(request));
-      ochusha_async_buffer_emit_access_failed(buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			message);
-#if 0
-      fprintf(stderr, "%s\n", message);
-#endif
-    }
-  else
-    ochusha_async_buffer_emit_access_failed(buffer,
-			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			  _("Access failed: unknown reason."));
-
-  if (request != NULL)
-    ghttp_request_destroy(request);
-  args->request = NULL;
-
-  return FALSE;
-
-}
-
-
-static void
-force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
-{
-  NetworkBrokerJobArgs *args;
-  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
-						    broker_id);
-  OchushaNetworkBrokerBufferStatus *status
-    = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
-
-#if DEBUG_THREAD_MOST
-  fprintf(stderr, "force_read_synchronously() thread is invoked.\n");
-#endif
-
-  if (!ochusha_async_buffer_active_ref(buffer))
-    {
-      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      goto finish_force_read;
-    }
-
-  args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
-  if (args == NULL)
-    {
-      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      goto finish_force_read;
-    }
-
-  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
-
-  if (args->if_modified_since != NULL)
-    {
-      G_FREE(args->if_modified_since);
-      args->if_modified_since = NULL;
-    }
-
-  if (http_read_from_url_synchronously(broker, buffer))
-    {
-      /* ̿ */
-      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_OK;
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "reading from %s succeeded.\n", args->url);
-#endif
-    }
-  else
-    {
-      /* ̿ */
-      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-#if DEBUG_NETWORK_MOST
-      fprintf(stderr, "reading from %s failed.\n", args->url);
-#endif
-    }
-
-  ochusha_async_buffer_active_unref(buffer);
-
- finish_force_read:
-  ochusha_async_buffer_fix(buffer);
-  g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(buffer);
-}
-
-
-/*
- * ochusha_network_broker_read_from_cgi
- *
- * Ϳ줿URLǡɤ߹ࡣ
- * offline⡼ɤǤNULL֤
- *
- * åˤϿʤޤCGIξHEAD쥹ݥ󥹤Ƥʤ
- * ȤΤǡƱ⡼ɤȤGETȯɤ߹ࡣ
- */
-OchushaAsyncBuffer *
-ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
-				     OchushaAsyncBuffer *buffer,
-				     const char *url)
-{
-  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
-		       && broker->config != NULL && url != NULL, NULL);
-#if DEBUG_NETWORK_MOST
-  fprintf(stderr, "ochusha_network_broker_read_from_cgi: %s\n", url);
-#endif
-
-  if (broker->config->offline)
-    return NULL;
-
-  return ochusha_network_broker_employ_worker_thread(broker, buffer, url, NULL,
-					(JobFunc *)force_read_synchronously,
-					FALSE, 0);
-}
-
-
 /*
  * ochusha_network_broker_get_response_header:
  *
@@ -2664,7 +2232,7 @@ ochusha_network_broker_get_header_names(OchushaNetworkBroker *broker,
 
 gboolean
 ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
-				char *url, const char *server,
+				const char *url, const char *server,
 				const char *referer,
 				const char *cookie, const char *body,
 				OchushaNetworkBrokerPostStatus *status)
@@ -2679,7 +2247,7 @@ ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
   ghttp_set_uri(request, url);
   ghttp_set_type(request, ghttp_type_post);
 
-  setup_common_request_headers(broker, request, TRUE);
+  setup_common_request_headers(broker, request, TRUE, TRUE);
   ghttp_set_header(request, http_hdr_Host, server);
 
   /* ɬפʤΤ */
@@ -2690,13 +2258,14 @@ ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
   ghttp_set_header(request, http_hdr_Accept_Charset, "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
 
   ghttp_set_header(request, http_hdr_Connection, "close");
-  ghttp_set_header(request, http_hdr_Referrer, referer);
-  ghttp_set_header(request, "Referer", referer);
+  ghttp_set_header(request, http_hdr_Referer, referer);
 
   if (cookie != NULL)
     ghttp_set_header(request, "Cookie", cookie);
 
-  snprintf(message, LOG_MESSAGE_SIZE, _("Posting a response: %s\n"), url);
+  snprintf(message, LOG_MESSAGE_SIZE, _("Posting a response to: %s\n"), url);
+  ochusha_network_broker_output_log(broker, message);
+  snprintf(message, LOG_MESSAGE_SIZE, _("Message Body: %s\n"), body);
   ochusha_network_broker_output_log(broker, message);
 
   ghttp_set_body(request, (char *)body, strlen(body));
@@ -2763,3 +2332,99 @@ ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
   return TRUE;
 }
 
+
+gboolean
+ochusha_network_broker_try_post_raw(OchushaNetworkBroker *broker,
+				    const char *url, const char *server,
+				    const char **headers, const char *body,
+				    OchushaNetworkBrokerPostStatus *status)
+{
+  ghttp_request *request = ghttp_request_new();
+  int i;
+#if DEBUG_NETWORK || DEBUG_POST
+  char message[LOG_MESSAGE_SIZE];
+#endif
+
+  g_return_val_if_fail(request != NULL, FALSE);
+
+  ghttp_set_uri(request, url);
+  ghttp_set_type(request, ghttp_type_post);
+
+  setup_common_request_headers(broker, request, TRUE, FALSE);
+  if (headers != NULL)
+    {
+      for (i = 0; headers[i] != NULL; i += 2)
+	ghttp_set_header(request, headers[i], headers[i + 1]);
+    }
+  ghttp_set_header(request, http_hdr_Host, server);
+
+  /* ɬפʤΤ */
+  ghttp_set_header(request, http_hdr_Accept_Charset, "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
+
+  ghttp_set_header(request, http_hdr_Connection, "close");
+
+  snprintf(message, LOG_MESSAGE_SIZE, _("Posting to: %s\n"), url);
+  ochusha_network_broker_output_log(broker, message);
+
+  ghttp_set_body(request, (char *)body, strlen(body));
+  ghttp_prepare(request);
+  ghttp_load_verify_locations(request, OCHUSHA_CA_BUNDLE_CRT, NULL);
+
+  if (ghttp_process(request) == ghttp_error)
+    {
+      ochusha_network_broker_output_log(broker, _("Posting failed: ghttp_process() returns ghttp_error\n"));
+#if DEBUG_POST_MOST
+      fprintf(stderr, "ghttp_process() returns ghttp_error\n");
+#endif
+      ghttp_request_destroy(request);
+      if (status != NULL)
+	{
+	  status->status_code = 0;
+	  status->body = NULL;
+	  status->set_cookie = NULL;
+	}
+
+      return FALSE;
+    }
+
+  if (status != NULL)
+    {
+      const char *set_cookie;
+      status->status_code = ghttp_status_code(request);
+      status->body = G_STRNDUP(ghttp_get_body(request),
+			       ghttp_get_body_len(request));
+      set_cookie = ghttp_get_header(request, http_hdr_Set_Cookie);
+      if (set_cookie != NULL)
+	status->set_cookie = G_STRDUP(set_cookie);
+      else
+	status->set_cookie = NULL;
+#if DEBUG_NETWORK
+      {
+	int i;
+	char **headers = NULL;
+	int num_headers = 0;
+	snprintf(message, LOG_MESSAGE_SIZE, "Status: %s (%d)\n",
+		 ghttp_reason_phrase(request), status->status_code);
+	ochusha_network_broker_output_log(broker, message);
+	ghttp_get_header_names(request, &headers, &num_headers);
+	for (i = 0; i < num_headers; i++)
+	  {
+	    snprintf(message, LOG_MESSAGE_SIZE, "%s: %s\n",
+		     headers[i], ghttp_get_header(request, headers[i]));
+	    ochusha_network_broker_output_log(broker, message);
+	    free(headers[i]);
+	  }
+	if (headers != NULL)
+	  free(headers);
+      }
+#endif
+    }
+
+  ghttp_request_destroy(request);
+
+#if DEBUG_POST
+  ochusha_network_broker_output_log(broker, _("Posting done.\n"));
+#endif
+
+  return TRUE;
+}
