@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.h,v 1.14 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_network_broker.h,v 1.18 2004/02/26 06:05:10 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_NETWORK_BROKER_H_
@@ -139,10 +139,14 @@ OchushaAsyncBuffer *ochusha_network_broker_read_from_url(
 					const char *if_modified_since,
 					OchushaNetworkBrokerCacheMode mode,
 					gboolean modestly, int chunksize);
-OchushaAsyncBuffer *ochusha_network_broker_read_from_cgi(
+
+OchushaAsyncBuffer *ochusha_network_broker_read_from_url_full(
 					OchushaNetworkBroker *broker,
 					OchushaAsyncBuffer *buffer,
-					const char *url);
+					const char *url, const char *cache_url,
+					const char *if_modified_since,
+					OchushaNetworkBrokerCacheMode mode,
+					gboolean modestly, int chunksize);
 
 const char *ochusha_network_broker_get_response_header(
 						  OchushaNetworkBroker *broker,
@@ -161,10 +165,14 @@ typedef struct _OchushaNetworkBrokerPostStatus
 
 
 gboolean ochusha_network_broker_try_post(OchushaNetworkBroker *broker,
-				char *url, const char *server,
+				const char *url, const char *server,
 				const char *referer,
 				const char *cookie, const char *body,
 				OchushaNetworkBrokerPostStatus *status);
+gboolean ochusha_network_broker_try_post_raw(OchushaNetworkBroker *broker,
+				const char *url, const char *server,
+				const char **headers, const char *body,
+				OchushaNetworkBrokerPostStatus *status);
 
 #ifdef __cplusplus
 } // extern "C"
