@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.64 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.69 2004/02/27 22:17:12 fuyu Exp $
  */
 
 #include "config.h"
@@ -34,9 +34,15 @@
 #include "ochusha_thread_2ch.h"
 #include "ochusha_utils_2ch.h"
 
+#include "htmlutils.h"
+#include "worker.h"
+
 #include <glib-object.h>
 #include <glib.h>
 
+#include <errno.h>
+#include <fcntl.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -72,12 +78,13 @@ static gboolean ochusha_thread_2ch_check_url(OchushaBBSThread *thread,
 					     unsigned int *to_p);
 static void ochusha_thread_2ch_remove_cache(OchushaBBSThread *thread,
 					    OchushaConfig *config);
-
 static OchushaAsyncBuffer *ochusha_thread_2ch_get_responses_source(
 					OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
 					OchushaAsyncBuffer *buffer,
 					OchushaNetworkBrokerCacheMode mode);
+static char *extract_2ch_response(char *start_pos, char *eol_pos,
+				  OchushaBBSResponse *response);
 static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   OchushaAsyncBuffer *buffer,
 						   int start, int number,
@@ -105,7 +112,8 @@ static gboolean ochusha_thread_2ch_post_supported(OchushaBBSThread *thread);
 
 static gboolean ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
-					const OchushaBBSResponse *response);
+					const OchushaBBSResponse *response,
+					gboolean use_id);
 
 static char *ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread,
 						     int from, int to);
@@ -240,6 +248,12 @@ ochusha_thread_2ch_finalize(GObject *object)
       thread->kako_html = NULL;
     }
 
+  if (thread->oyster_url != NULL)
+    {
+      G_FREE(thread->oyster_url);
+      thread->oyster_url = NULL;
+    }
+
   if (thread->responses != NULL)
     {
       G_FREE(thread->responses);
@@ -316,6 +330,9 @@ ochusha_thread_2ch_write_threadlist_element(OchushaBBSThread *thread,
 {
   OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
+  if (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH_HEADLINE)
+    thread->number_of_responses_read = 0;
+
   if (parent_class->write_threadlist_element != NULL)
     (*parent_class->write_threadlist_element)(thread, threadlist_xml);
 
@@ -540,6 +557,379 @@ ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
 }
 
 
+static const char *
+ochusha_thread_2ch_get_oyster_url(OchushaThread2ch *thread_2ch,
+				  const char *sid)
+{
+  OchushaBulletinBoard *board;
+  OchushaBBSThread *thread;
+  char url_buffer[PATH_MAX];
+
+  if (thread_2ch->oyster_url != NULL)
+    return thread_2ch->oyster_url;
+
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
+
+  thread = OCHUSHA_BBS_THREAD(thread_2ch);
+  board = ochusha_bbs_thread_get_board(thread);
+  g_return_val_if_fail(board->bbs_type == OCHUSHA_BBS_TYPE_2CH, NULL);
+
+  snprintf(url_buffer, PATH_MAX,
+	   "http://%s/test/offlaw.cgi/%s/%s/?raw=0.0&sid=%s",
+	   ochusha_bulletin_board_get_server(board),
+	   ochusha_bulletin_board_get_id(board),
+	   ochusha_bbs_thread_get_id(thread),
+	   sid);
+  thread_2ch->oyster_url = G_STRDUP(url_buffer);
+  return thread_2ch->oyster_url;
+}
+
+
+typedef struct _ProcessOfflawResultJobArgs
+{
+  OchushaAsyncBuffer *dat_buffer;	/* ʪDATǡǼ */
+  OchushaAsyncBuffer *cgi_buffer;	/* offlaw.cgiη̤Ǽ */
+  OchushaBBSThread *thread;
+  OchushaNetworkBroker *broker;
+} ProcessOfflawResultJobArgs;
+
+
+static void
+process_offlaw_result(WorkerThread *employee, ProcessOfflawResultJobArgs *args)
+{
+  OchushaAsyncBuffer *dat_buffer = args->dat_buffer;
+  OchushaAsyncBuffer *cgi_buffer = args->cgi_buffer;
+  OchushaBBSThread *thread = args->thread;
+  OchushaNetworkBroker *broker = args->broker;
+  size_t offset = 0;
+  char *cur_pos;
+  char *eol_pos;
+  char *title_pos;
+  int fd;
+  const char *dat_url;
+  gchar message[4096];
+  OchushaBBSResponse response = { NULL, NULL, NULL, NULL };
+
+  if (!ochusha_async_buffer_active_ref(cgi_buffer))
+    {
+      goto terminated;
+    }
+
+  if (!ochusha_async_buffer_active_ref(dat_buffer))
+    {
+      ochusha_async_buffer_active_unref(cgi_buffer);
+      goto terminated;
+    }
+
+  ochusha_async_buffer_lock(cgi_buffer);
+  /* 1ܤɤǡofflaw.cgiη̤Ĵ٤롣 */
+  while ((eol_pos = memchr((const char *)cgi_buffer->buffer, '\n',
+			   cgi_buffer->length))
+	 == NULL)
+    if (cgi_buffer->fixed || !ochusha_async_buffer_wait(cgi_buffer))
+      {
+#if DEBUG_ASYNC_BUFFER_MOST
+	fprintf(stderr, "process_offlaw_result(): buffer has been terminated.\n");
+#endif
+	ochusha_async_buffer_unlock(cgi_buffer);
+	ochusha_async_buffer_active_unref(cgi_buffer);
+	ochusha_async_buffer_active_unref(dat_buffer);
+	goto terminated;
+      }
+
+  /* ޤ褿顢offlaw.cgiη̤1ܤɤ߹ߺ */
+  if (cgi_buffer->length >= 3 && strncmp((const char *)cgi_buffer->buffer,
+					 "+OK", 3) == 0)
+    {
+      /* eol_pos + 1ʹߤDATե */
+      cur_pos = eol_pos + 1;
+      offset = cur_pos - cgi_buffer->buffer;
+      eol_pos = NULL;
+    }
+  else
+    {
+      char *tmp_pos;
+      *eol_pos = '\0';
+      eol_pos = NULL;
+      /* 顼 */
+      /* (HTML)ҸˤäƤ뤫åIDδڤΤߡ */
+      tmp_pos = strstr((const char *)cgi_buffer->buffer, "kako/");
+      if (tmp_pos != NULL)
+	{
+	  /* (HTML)ҸˤäƤ */
+	  OchushaAsyncBuffer *old_buffer = cgi_buffer;
+	  char kako_url[PATH_MAX];
+	  snprintf(kako_url, PATH_MAX, "%s%s",
+		   ochusha_bulletin_board_get_base_url(thread->board),
+		   tmp_pos);
+	  /* cgi_bufferkako_url
+	   * ɤ߹ޤǡΤbuffer֤
+	   */
+	  cgi_buffer = ochusha_network_broker_read_from_url(broker, NULL,
+					kako_url, NULL,
+					OCHUSHA_NETWORK_BROKER_CACHE_IGNORE,
+					FALSE,
+					broker->config->thread_chunksize);
+	  if (cgi_buffer == NULL)
+	    {
+	      /*  */
+	      cgi_buffer = old_buffer;
+	      ochusha_async_buffer_unlock(cgi_buffer);
+	      ochusha_async_buffer_active_unref(cgi_buffer);
+	      ochusha_async_buffer_active_unref(dat_buffer);
+	      goto terminated;
+	    }
+	  ochusha_async_buffer_unlock(old_buffer);
+	  ochusha_async_buffer_active_unref(old_buffer);
+	  OCHU_OBJECT_UNREF(old_buffer);
+
+	  if (!ochusha_async_buffer_active_ref(cgi_buffer))
+	    {
+	      ochusha_async_buffer_active_unref(dat_buffer);
+	      goto terminated;
+	    }
+	  ochusha_async_buffer_lock(cgi_buffer);
+
+	  g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_started",
+			G_CALLBACK(ochusha_async_buffer_emit_access_started),
+			dat_buffer);
+	  g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_progressed",
+			G_CALLBACK(ochusha_async_buffer_emit_access_progressed),
+			dat_buffer);
+	  g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_finished",
+			G_CALLBACK(ochusha_async_buffer_emit_access_finished),
+			dat_buffer);
+	  g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_failed",
+			G_CALLBACK(ochusha_async_buffer_emit_access_failed),
+			dat_buffer);
+
+	  cur_pos = NULL;
+	  /* HTMLѲѤthread򹹿 */
+	  tmp_pos = g_strrstr(kako_url, ".dat");
+	  if (tmp_pos != NULL)
+	    {
+	      *tmp_pos = '\0';
+	      g_strlcat(kako_url, ".html", PATH_MAX);
+	      ochusha_thread_2ch_set_kako_html_url(OCHUSHA_THREAD_2CH(thread),
+						   kako_url);
+	    }
+#if 0
+	  else
+	    {
+	      /* ä֤뤷ʤ */
+	    }
+#endif
+	}
+      else
+	{
+	  /* Υ顼(¿ʬåIDδڤ) */
+	  const char *encoding
+	    = ochusha_bbs_thread_get_response_character_encoding(thread);
+	  iconv_helper *helper
+	    = ochusha_bbs_thread_get_response_iconv_helper(thread);
+	  iconv_t converter = iconv_open(helper ? "UTF-8" : "UTF-8//IGNORE",
+					 encoding);
+	  if (converter != (iconv_t)-1)
+	    {
+	      gchar *result = convert_string(converter, helper,
+					     (const char *)cgi_buffer->buffer,
+					     -1);
+
+	      snprintf(message, 4096, _("Error by offlaw.cgi: %s\n"), result);
+	      ochusha_network_broker_output_log(broker, message);
+	      G_FREE(result);
+	      iconv_close(converter);
+	    }
+	  else
+	    ochusha_network_broker_output_log(broker,
+					      _("Error by offlaw.cgi"));
+
+	  ochusha_async_buffer_unlock(cgi_buffer);
+	  ochusha_async_buffer_active_unref(cgi_buffer);
+	  ochusha_async_buffer_active_unref(dat_buffer);
+	  goto terminated;
+	}
+    }
+
+  /* DATǡ饹쥿ơOchushaBBSThreadtitle򹹿 */
+  while ((eol_pos = memchr((const char *)cgi_buffer->buffer + offset,
+			   '\n', cgi_buffer->length)) == NULL)
+    if (cgi_buffer->fixed || !ochusha_async_buffer_wait(cgi_buffer))
+      {
+#if DEBUG_ASYNC_BUFFER_MOST
+	fprintf(stderr, "process_offlaw_result(): buffer has been terminated.\n");
+#endif
+	ochusha_async_buffer_unlock(cgi_buffer);
+	ochusha_async_buffer_active_unref(cgi_buffer);
+	ochusha_async_buffer_active_unref(dat_buffer);
+	goto terminated;
+      }
+  cur_pos = (char *)cgi_buffer->buffer + offset;
+
+  title_pos = extract_2ch_response(cur_pos, eol_pos, &response);
+  if (title_pos != NULL)
+    {
+      const char *encoding
+	= ochusha_bbs_thread_get_response_character_encoding(thread);
+      iconv_helper *helper
+	= ochusha_bbs_thread_get_response_iconv_helper(thread);
+      iconv_t converter = iconv_open(helper ? "UTF-8" : "UTF-8//IGNORE",
+				     encoding);
+      if (converter != (iconv_t)-1)
+	{
+	  gchar *title = simple_string_canon(title_pos, eol_pos - title_pos,
+					     converter, helper);
+	  ochusha_bbs_thread_set_title(thread, title);
+	  G_FREE(title);
+	  iconv_close(converter);
+	}
+    }
+
+  /* DATǡΤҤdat_buffer˥ԡ */
+  while (TRUE)
+    {
+      if (cgi_buffer->length > offset)
+	{
+	  ochusha_async_buffer_append_data(dat_buffer,
+					   (const char *)(cgi_buffer->buffer
+							  + offset),
+					   cgi_buffer->length - offset);
+	  offset = cgi_buffer->length;
+	}
+
+      if (cgi_buffer->fixed)
+	break;
+
+      if (!ochusha_async_buffer_wait(cgi_buffer))
+	{
+#if DEBUG_ASYNC_BUFFER_MOST
+	  fprintf(stderr, "process_offlaw_result(): buffer has been terminated.\n");
+#endif
+	  ochusha_async_buffer_unlock(cgi_buffer);
+	  ochusha_async_buffer_active_unref(cgi_buffer);
+	  ochusha_async_buffer_active_unref(dat_buffer);
+	  goto terminated;
+	}
+    }
+  ochusha_async_buffer_unlock(cgi_buffer);
+
+  ochusha_async_buffer_active_unref(cgi_buffer);
+  ochusha_async_buffer_active_unref(dat_buffer);
+
+  ochusha_async_buffer_fix(dat_buffer);
+
+  /* ХåեƤ򥭥å˽񤭹 */
+  dat_url = ochusha_thread_2ch_get_dat_url(OCHUSHA_THREAD_2CH(thread));
+  fd = ochusha_config_cache_open_file(broker->config, dat_url,
+				      O_WRONLY | O_TRUNC | O_CREAT);
+  if (fd >= 0)
+    {
+      ssize_t len = write(fd, (void *)dat_buffer->buffer, dat_buffer->length);
+      close(fd);
+      if (len == dat_buffer->length)
+	{
+	  /* ｪλ */
+	  if ((thread->flags & OCHUSHA_BBS_THREAD_KAKO) == 0)
+	    thread->flags |= OCHUSHA_BBS_THREAD_DAT_OYSTER;
+	}
+      else
+	{
+	  ochusha_config_cache_unlink_file(broker->config, dat_url);
+	  fd = -1;
+	}
+#if DEBUG_NETWORK_MOST
+      if (fd < 0)
+	fprintf(stderr, "Couldn't write cache file for %s: %s (%d)\n",
+		dat_url, strerror(errno), errno);
+#endif	
+    }
+
+ terminated:
+  OCHU_OBJECT_UNREF(cgi_buffer);
+  OCHU_OBJECT_UNREF(dat_buffer);
+  OCHU_OBJECT_UNREF(broker);
+}
+
+
+static void
+buffer_status_free(OchushaNetworkBrokerBufferStatus *status)
+{
+  if (status->last_modified != NULL)
+    G_FREE(status->last_modified);
+  if (status->date != NULL)
+    G_FREE(status->date);
+  G_FREE(status);
+}
+
+
+static OchushaAsyncBuffer *
+ochusha_thread_2ch_read_dat_by_offlaw(OchushaThread2ch *thread_2ch,
+				      OchushaNetworkBroker *broker,
+				      OchushaAsyncBuffer *buffer)
+{
+  OchushaAsyncBuffer *dat_buffer = buffer;
+  OchushaAsyncBuffer *cgi_buffer;
+  OchushaNetworkBrokerBufferStatus *status;
+  const char *url = ochusha_thread_2ch_get_oyster_url(thread_2ch,
+						      broker->config->session_id_2ch);
+
+  if (dat_buffer != NULL && !ochusha_async_buffer_reset(dat_buffer))
+    {
+      OCHU_OBJECT_UNREF(dat_buffer);
+      dat_buffer = NULL;
+    }
+
+  if (dat_buffer == NULL)
+    dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+
+  status = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
+  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY;
+  g_object_set_data_full(G_OBJECT(dat_buffer),
+		"OchushaNetworkBroker::BufferStatus", status,
+		(GDestroyNotify)buffer_status_free);
+
+  cgi_buffer = ochusha_network_broker_read_from_url(broker, NULL, url, NULL,
+					OCHUSHA_NETWORK_BROKER_CACHE_IGNORE,
+					FALSE,
+					broker->config->thread_chunksize);
+  if (cgi_buffer != NULL)
+    {
+      /* åɵư */
+      WorkerJob *job = G_NEW0(WorkerJob, 1);
+      ProcessOfflawResultJobArgs *args = G_NEW0(ProcessOfflawResultJobArgs, 1);
+
+      g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_started",
+			G_CALLBACK(ochusha_async_buffer_emit_access_started),
+			dat_buffer);
+      g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_progressed",
+			G_CALLBACK(ochusha_async_buffer_emit_access_progressed),
+			dat_buffer);
+      g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_finished",
+			G_CALLBACK(ochusha_async_buffer_emit_access_finished),
+			dat_buffer);
+      g_signal_connect_swapped(G_OBJECT(cgi_buffer), "access_failed",
+			G_CALLBACK(ochusha_async_buffer_emit_access_failed),
+			dat_buffer);
+
+      args->dat_buffer = dat_buffer;
+      args->cgi_buffer = cgi_buffer;
+      args->thread = OCHUSHA_BBS_THREAD(thread_2ch);
+      args->broker = broker;
+
+      job->canceled = FALSE;
+      job->job = (JobFunc *)process_offlaw_result;
+      job->args = args;
+
+      OCHU_OBJECT_REF(dat_buffer);
+      OCHU_OBJECT_REF(broker);
+
+      commit_job(job);
+    }
+
+  return dat_buffer;
+}
+
+
 static OchushaAsyncBuffer *
 ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 					OchushaNetworkBroker *broker,
@@ -547,22 +937,48 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 					OchushaNetworkBrokerCacheMode mode)
 {
   OchushaThread2ch *thread_2ch;
+  const char *url;
+  const char *cache_url = NULL;
+
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread)
 		       && OCHUSHA_IS_NETWORK_BROKER(broker), NULL);
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
-  if (thread->flags & OCHUSHA_BBS_THREAD_KAKO)
+  url = ochusha_thread_2ch_get_dat_url(thread_2ch);
+
+  if (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH_HEADLINE)
+    mode = OCHUSHA_NETWORK_BROKER_CACHE_IGNORE;
+  else if (thread->flags & OCHUSHA_BBS_THREAD_KAKO)
     mode = OCHUSHA_NETWORK_BROKER_CACHE_AS_IS;
-  else if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED
-	   || ((thread->flags & OCHUSHA_BBS_THREAD_STOPPED)
-	       && thread->number_of_responses_read > 0))
+  else if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
+    {
+      if (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH
+	  && broker->config->login_2ch
+	  && (thread->flags & OCHUSHA_BBS_THREAD_DAT_OYSTER) == 0)
+	{
+	  /* ɤ */
+	  return ochusha_thread_2ch_read_dat_by_offlaw(thread_2ch, broker,
+						       buffer);
+	}
+      else
+	{
+	  /*
+	   * loginƤʤˡֲҸˤURLͽۤɤǤߤפä
+	   * ΤϤϤݤʤΤǡ褺󤷡
+	   */
+	  mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
+	}
+    }
+  else if ((thread->flags & OCHUSHA_BBS_THREAD_STOPPED)
+	   && thread->number_of_responses_read <= thread->number_of_responses_on_server)
     mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
 
-  return ochusha_network_broker_read_from_url(broker, buffer,
-				ochusha_thread_2ch_get_dat_url(thread_2ch),
-				thread_2ch->last_modified, mode, FALSE,
-				broker->config->thread_chunksize);
+  return ochusha_network_broker_read_from_url_full(broker, buffer,
+					url, cache_url,
+					thread_2ch->last_modified,
+					mode, FALSE,
+					broker->config->thread_chunksize);
 }
 
 
@@ -819,7 +1235,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
        * 1000쥹ʬǤ⡹4000ХȤʤΤǡ
        * ǽ餫MAX_RESPONSE쥹ʬ϶Ȥ롣
        */
-      responses = G_NEW0(unsigned int, MAX_RESPONSE);
+      responses = G_MALLOC0(sizeof(unsigned int) * MAX_RESPONSE);
       thread_2ch->responses = responses;
       thread_2ch->responses_length = MAX_RESPONSE;
     }
@@ -841,7 +1257,12 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
     rest_of_responses = number;
     offset = 0;
     i = 0;
-    if (status->state != OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
+    if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
+      {
+	memset(thread_2ch->responses, 0,
+	       thread_2ch->responses_length * sizeof(unsigned int));
+      }
+    else
       {
 	if (start > 1 && start < thread_2ch->responses_length)
 	  {
@@ -887,12 +1308,13 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 
 	    if (i >= thread_2ch->responses_length)
 	      {
-		guint new_len = thread_2ch->responses_length * 2;
-		thread_2ch->responses = G_REALLOC(thread_2ch->responses,
-						  new_len);
+		unsigned int new_len = thread_2ch->responses_length * 2;
+		thread_2ch->responses
+		  = G_REALLOC(thread_2ch->responses,
+			      new_len * sizeof(unsigned int));
 		memset(thread_2ch->responses + thread_2ch->responses_length, 0,
 		       (new_len - thread_2ch->responses_length)
-		       * sizeof(guint));
+		       * sizeof(unsigned int));
 		thread_2ch->responses_length = new_len;
 		responses = thread_2ch->responses;
 	      }
@@ -1065,7 +1487,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    status->state
 	      = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
 	    memset(thread_2ch->responses, 0,
-		   thread_2ch->responses_length * sizeof(guint));
+		   thread_2ch->responses_length * sizeof(unsigned int));
 	    goto retry_analysis;
 	  }
       }
@@ -1102,6 +1524,9 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   if (end_parsing_cb != NULL)
     (*end_parsing_cb)(callback_data);
 
+  if (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH_HEADLINE)
+    thread->number_of_responses_read = 0;
+
   return TRUE;
 }
 
@@ -1136,6 +1561,7 @@ ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread, int from,
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CH_COMPATIBLE:
+    case OCHUSHA_BBS_TYPE_2CH_HEADLINE:
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
       {
 	if (from >= to)
@@ -1195,14 +1621,15 @@ ochusha_thread_2ch_post_supported(OchushaBBSThread *thread)
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), FALSE);
   g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(thread->board), FALSE);
 
-  return TRUE;
+  return thread->board->bbs_type != OCHUSHA_BBS_TYPE_2CH_HEADLINE;
 }
 
 
 static gboolean
 ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 				 OchushaNetworkBroker *broker,
-				 const OchushaBBSResponse *response)
+				 const OchushaBBSResponse *response,
+				 gboolean use_id)
 {
   OchushaThread2ch *thread_2ch;
   OchushaThread2chClass *thread_2ch_class;
@@ -1281,7 +1708,12 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 							    message, bbs,
 							    key, time);
   else
-    query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&time=%ld", from, mail, message, bbs, key, time);
+    {
+      if (use_id && broker->config->login_2ch)
+	query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&sid=%s&time=%ld", from, mail, message, bbs, key, broker->config->session_id_2ch, time);
+      else
+	query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&time=%ld", from, mail, message, bbs, key, time);
+    }
 
   if (query == NULL)
     goto error_exit;
@@ -1289,8 +1721,24 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
   if (post_result == OCHUSHA_UTILS_2CH_POST_NO_COOKIE)
     {
-      sleep(10);
-      post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
+      if (ochusha_board_2ch_get_cookie(board_2ch) != NULL)
+	{
+	  if (thread->board->bbs_type == OCHUSHA_BBS_TYPE_2CH)
+	    {
+	      g_free(query);
+	      if (use_id && broker->config->login_2ch)
+		query = g_strdup_printf("submit=%%91%%53%%90%%D3%%94%%43%%82%%F0%%95%%89%%82%%A4%%82%%B1%%82%%C6%%82%%F0%%8F%%B3%%91%%F8%%82%%B5%%82%%C4%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&sid=%s&time=%ld", from, mail, message, bbs, key, broker->config->session_id_2ch, time);
+	      else
+		query = g_strdup_printf("submit=%%91%%53%%90%%D3%%94%%43%%82%%F0%%95%%89%%82%%A4%%82%%B1%%82%%C6%%82%%F0%%8F%%B3%%91%%F8%%82%%B5%%82%%C4%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&time=%ld", from, mail, message, bbs, key, time);
+	    }
+	  sleep(10);
+	  post_result = ochusha_utils_2ch_try_post(broker, thread->board,
+						   query);
+	}
+      else
+	{
+	  ochusha_network_broker_output_log(broker, "No Cookie!\n");
+	}
     }
 
 #if 0
