@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.34 2004/01/23 18:52:02 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.37 2004/02/27 22:17:12 fuyu Exp $
  */
 
 #include "config.h"
@@ -533,7 +533,7 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
        * 1000쥹ʬǤ⡹4000ХȤʤΤǡ
        * ǽ餫MAX_RESPONSE쥹ʬ϶Ȥ롣
        */
-      responses = G_NEW0(unsigned int, MAX_RESPONSE);
+      responses = G_MALLOC0(sizeof(unsigned int) * MAX_RESPONSE);
       thread_2ch->responses = responses;
       thread_2ch->responses_length = MAX_RESPONSE;
     }
@@ -555,7 +555,12 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
     rest_of_responses = number;
     offset = 0;
     i = 0;
-    if (status->state != OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
+    if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
+      {
+	memset(thread_2ch->responses, 0,
+	       thread_2ch->responses_length * sizeof(unsigned int));
+      }
+    else
       {
 	if (start > 1 && start < thread_2ch->responses_length)
 	  {
@@ -602,11 +607,12 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	    if (i >= thread_2ch->responses_length)
 	      {
 		guint new_len = thread_2ch->responses_length * 2;
-		thread_2ch->responses = G_REALLOC(thread_2ch->responses,
-						  new_len);
+		thread_2ch->responses
+		  = G_REALLOC(thread_2ch->responses,
+			      new_len * sizeof(unsigned int));
 		memset(thread_2ch->responses + thread_2ch->responses_length, 0,
 		       (new_len - thread_2ch->responses_length)
-		       * sizeof(guint));
+		       * sizeof(unsigned int));
 		thread_2ch->responses_length = new_len;
 		responses = thread_2ch->responses;
 	      }
@@ -1531,6 +1537,8 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   ochusha_async_buffer_active_unref(html_buffer);
   ochusha_async_buffer_active_unref(dat_buffer);
 
+  ochusha_async_buffer_fix(dat_buffer);
+
   /* ɬפʤХåեƤ򥭥å˽񤭹 */
   if (dat_buffer->length > incoming_offset)
     {
@@ -1550,15 +1558,13 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	      fd = -1;
 	    }
 	}
-#if DEBUG_NETWORK
+#if DEBUG_NETWORK_MOST
       if (fd < 0)
 	fprintf(stderr, "Couldn't update cache file for %s: %s (%d)\n",
 		fake_dat_url, strerror(errno), errno);
 #endif	      
     }
 
-  ochusha_async_buffer_fix(dat_buffer);
-
  terminated:
   if (title_pattern != NULL)
     regex_free(title_pattern);
@@ -1709,12 +1715,9 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 #endif
 
   if (cgi_url[0] != '\0')
-#if 1	/* ʤǤΤasync⡼ɤǥȼԤ롣*/
-    html_buffer = ochusha_network_broker_read_from_cgi(broker, NULL, cgi_url);
-#else
     html_buffer = ochusha_network_broker_read_from_url(broker, NULL, cgi_url,
-						       NULL, mode);
-#endif
+					NULL, mode, FALSE,
+					broker->config->thread_chunksize);
   else
     html_buffer = NULL;
 
