@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.h,v 1.9 2004/01/15 02:12:14 fuyu Exp $
+ * $Id: ochusha_utils_2ch.h,v 1.10 2004/01/30 22:55:47 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UTILS_2CH_H_
@@ -78,7 +78,10 @@ OchushaUtils2chPostResult ochusha_utils_2ch_try_post(
 						OchushaNetworkBroker *broker,
 						OchushaBulletinBoard *board,
 						const char *message);
-
+void ochusha_utils_2ch_login(OchushaConfig *config,
+			     OchushaNetworkBroker *broker,
+			     const char *user_id, const char *passwd);
+void ochusha_utils_2ch_logout(OchushaConfig *config);
 
 #ifdef __cplusplus
 } // extern "C"
