@@ -7,12 +7,6 @@
 **********************************************************************/
 #include "regint.h"
 
-static UChar*
-get_right_adjust_char_head_with_prev(RegCharEncoding code,
-				     UChar* start, UChar* s, UChar** prev);
-static UChar*
-step_backward_char(RegCharEncoding code, UChar* start, UChar* s, int n);
-
 static void
 region_list_clear(RegRegion** list)
 {
@@ -715,35 +709,47 @@ stack_double(StackType** arg_stk_base, StackType** arg_stk_end,
 } while(0)
 
 
-#define CASETABLE_TOLOWER(c)   (casetable[c])
-
-/* byte_code is already converted to lower-case at string compile time */
-#define SBTRANSCMP(byte_code,c)    (byte_code == CASETABLE_TOLOWER(c))
-
 #define STRING_CMP(s1,s2,len) do {\
   while (len-- > 0) {\
     if (*s1++ != *s2++) goto fail;\
   }\
 } while(0)
 
-#define STRING_CMP_IC(s1,s2,len) do {\
-  int slen; \
-  while (len > 0) {\
-    slen = mblen(encode, *s1); \
-    if (slen == 1) {\
-      if (CASETABLE_TOLOWER(*s1) != CASETABLE_TOLOWER(*s2)) \
-        goto fail;\
-      s1++; s2++; len--; \
-    } \
-    else {\
-      len -= slen; \
-      while (slen-- > 0) { \
-        if (*s1++ != *s2++) goto fail;\
-      } \
-    }\
-  }\
+#define STRING_CMP_IC(s1,ps2,len) do {\
+  if (string_cmp_ic(encode, s1, ps2, len) == 0) \
+    goto fail; \
 } while(0)
 
+static int string_cmp_ic(RegCharEncoding enc,
+			 UChar* s1, UChar** ps2, int mblen)
+{
+  UChar buf1[ENC_MBC_TO_LOWER_MAXLEN];
+  UChar buf2[ENC_MBC_TO_LOWER_MAXLEN];
+  UChar *p1, *p2, *end, *s2;
+  int len1, len2;
+
+  s2  = *ps2;
+  end = s1 + mblen;
+  while (s1 < end) {
+    len1 = ENC_MBC_TO_LOWER(enc, s1, buf1);
+    len2 = ENC_MBC_TO_LOWER(enc, s2, buf2);
+    if (len1 != len2) return 0;
+    p1 = buf1;
+    p2 = buf2;
+    while (len1-- > 0) {
+      if (*p1 != *p2) return 0;
+      p1++;
+      p2++;
+    }
+
+    s1 += enc_len(enc, *s1);
+    s2 += enc_len(enc, *s2);
+  }
+
+  *ps2 = s2;
+  return 1;
+}
+
 #define STRING_CMP_VALUE(s1,s2,len,is_fail) do {\
   is_fail = 0;\
   while (len-- > 0) {\
@@ -753,27 +759,11 @@ stack_double(StackType** arg_stk_base, StackType** arg_stk_end,
   }\
 } while(0)
 
-#define STRING_CMP_VALUE_IC(s1,s2,len,is_fail) do {\
-  int slen; \
-  is_fail = 0;\
-  while (len > 0) {\
-    slen = mblen(encode, *s1); \
-    if (slen == 1) {\
-      if (CASETABLE_TOLOWER(*s1) != CASETABLE_TOLOWER(*s2)) {\
-        is_fail = 1; break;\
-      }\
-      s1++; s2++; len--; \
-    } \
-    else {\
-      len -= slen; \
-      while (slen-- > 0) { \
-        if (*s1++ != *s2++) {\
-          is_fail = 1; break;\
-        }\
-      } \
-      if (is_fail != 0) break;\
-    }\
-  }\
+#define STRING_CMP_VALUE_IC(s1,ps2,len,is_fail) do {\
+  if (string_cmp_ic(encode, s1, ps2, len) == 0) \
+    is_fail = 1; \
+  else \
+    is_fail = 0; \
 } while(0)
 
 #define ON_STR_BEGIN(s)  ((s) == str)
@@ -875,24 +865,24 @@ regex_print_statistics(FILE* f)
 #endif
 
 extern int
-regex_is_in_wc_range(UChar* p, WCINT wc)
+regex_is_in_code_range(UChar* p, CodePoint code)
 {
-  WCINT n, *data;
+  CodePoint n, *data;
   int low, high, x;
 
-  GET_WCINT(n, p);
-  data = (WCINT* )p;
+  GET_CODE_POINT(n, p);
+  data = (CodePoint* )p;
   data++;
 
   for (low = 0, high = n; low < high; ) {
     x = (low + high) >> 1;
-    if (wc > data[x * 2 + 1])
+    if (code > data[x * 2 + 1])
       low = x + 1;
     else
       high = x;
   }
 
-  return ((low < n && wc >= data[low * 2]) ? 1 : 0);
+  return ((low < n && code >= data[low * 2]) ? 1 : 0);
 }
 
 
@@ -918,7 +908,6 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
   RelAddrType addr;
   RegOptionType option = reg->options;
   RegCharEncoding encode = reg->enc;
-  unsigned char* casetable = DefaultTransTable;
   int ignore_case;
   UChar *s, *q, *sbegin;
   UChar *p = reg->p;
@@ -963,7 +952,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       fprintf(stderr, "%4d> \"", (int )(s - str));
       bp = buf;
       for (i = 0, q = s; i < 7 && q < end; i++) {
-	len = mblen(encode, *q);
+	len = enc_len(encode, *q);
 	while (len-- > 0) *bp++ = *q++;
       }
       if (q < end) { xmemcpy(bp, "...\"", 4); bp += 4; }
@@ -984,6 +973,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	RegRegion* region = msa->region;
 	best_len = n;
 	if (region) {
+#ifdef USE_POSIX_REGION_OPTION
 	  if (IS_POSIX_REGION(msa->options)) {
 	    posix_regmatch_t* rmt = (posix_regmatch_t* )region;
 
@@ -1006,6 +996,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	    }
 	  }
 	  else {
+#endif /* USE_POSIX_REGION_OPTION */
 	    region->beg[0] = sstart - str;
 	    region->end[0] = s      - str;
 	    for (i = 1; i <= num_mem; i++) {
@@ -1045,7 +1036,9 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 		}
 	      }
 	    } /* list of captures */
+#ifdef USE_POSIX_REGION_OPTION
 	  } /* else IS_POSIX_REGION() */
+#endif
 	} /* if (region) */
       } /* n > best_len */
       STAT_OP_OUT;
@@ -1078,9 +1071,19 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       break;
 
     case OP_EXACT1_IC:  STAT_OP_IN(OP_EXACT1_IC);
-      if (! SBTRANSCMP(*p, *s)) goto fail;
-      DATA_ENSURE(1);
-      p++; s++;
+      {
+	int len;
+	UChar *q, lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
+
+	len = ENC_MBC_TO_LOWER(encode, s, lowbuf);
+	DATA_ENSURE(len);
+	q = lowbuf;
+	s += enc_len(encode, *s);
+	while (len-- > 0) {
+	  if (*p != *q) goto fail;
+	  p++; q++;
+	}
+      }
       STAT_OP_OUT;
       break;
 
@@ -1152,13 +1155,26 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       break;
 
     case OP_EXACTN_IC:  STAT_OP_IN(OP_EXACTN_IC);
-      GET_LENGTH_INC(tlen, p);
-      DATA_ENSURE(tlen);
-      while (tlen-- > 0) {
-	if (! SBTRANSCMP(*p, *s)) goto fail;
-	p++; s++;
+      {
+	int len;
+	UChar *q, *endp, lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
+
+	GET_LENGTH_INC(tlen, p);
+	endp = p + tlen;
+
+	while (p < endp) {
+	  len = ENC_MBC_TO_LOWER(encode, s, lowbuf);
+	  DATA_ENSURE(len);
+	  sprev = s;
+	  s += enc_len(encode, *s);
+	  q = lowbuf;
+	  while (len-- > 0) {
+	    if (*p != *q) goto fail;
+	    p++; q++;
+	  }
+	}
       }
-      sprev = s - 1;
+
       STAT_OP_OUT;
       continue;
       break;
@@ -1254,31 +1270,31 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       DATA_ENSURE(1);
       if (BITSET_AT(((BitSetRef )p), *s) == 0) goto fail;
       p += SIZE_BITSET;
-      s += mblen(encode, *s);   /* OP_CCLASS can match mb-code. \D, \S */
+      s += enc_len(encode, *s);   /* OP_CCLASS can match mb-code. \D, \S */
       STAT_OP_OUT;
       break;
 
     case OP_CCLASS_MB:  STAT_OP_IN(OP_CCLASS_MB);
-      if (! ismb(encode, *s)) goto fail;
+      if (! ENC_IS_MBC_HEAD(encode, *s)) goto fail;
 
     cclass_mb:
       GET_LENGTH_INC(tlen, p);
       {
-	WCINT wc;
+	CodePoint code;
 	UChar *ss;
-	int mb_len = mblen(encode, *s);
+	int mb_len = enc_len(encode, *s);
 
 	DATA_ENSURE(mb_len);
 	ss = s;
 	s += mb_len;
-	wc = MB2WC(ss, s, encode);
+	code = ENC_MBC_TO_CODE(encode, ss, s);
 
 #ifdef UNALIGNED_WORD_ACCESS
-	if (! regex_is_in_wc_range(p, wc)) goto fail;
+	if (! regex_is_in_code_range(p, code)) goto fail;
 #else
 	q = p;
 	ALIGNMENT_RIGHT(q);
-	if (! regex_is_in_wc_range(q, wc)) goto fail;
+	if (! regex_is_in_code_range(q, code)) goto fail;
 #endif
       }
       p += tlen;
@@ -1287,7 +1303,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
     case OP_CCLASS_MIX:  STAT_OP_IN(OP_CCLASS_MIX);
       DATA_ENSURE(1);
-      if (ismb(encode, *s)) {
+      if (ENC_IS_MBC_HEAD(encode, *s)) {
 	p += SIZE_BITSET;
 	goto cclass_mb;
       }
@@ -1307,12 +1323,12 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       DATA_ENSURE(1);
       if (BITSET_AT(((BitSetRef )p), *s) != 0) goto fail;
       p += SIZE_BITSET;
-      s += mblen(encode, *s);
+      s += enc_len(encode, *s);
       STAT_OP_OUT;
       break;
 
     case OP_CCLASS_MB_NOT:  STAT_OP_IN(OP_CCLASS_MB_NOT);
-      if (! ismb(encode, *s)) {
+      if (! ENC_IS_MBC_HEAD(encode, *s)) {
 	DATA_ENSURE(1);
 	s++;
 	GET_LENGTH_INC(tlen, p);
@@ -1323,9 +1339,9 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
     cclass_mb_not:
       GET_LENGTH_INC(tlen, p);
       {
-	WCINT wc;
+	CodePoint code;
 	UChar *ss;
-	int mb_len = mblen(encode, *s);
+	int mb_len = enc_len(encode, *s);
 
 	if (s + mb_len > end) {
 	  s = end;
@@ -1335,14 +1351,14 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
 	ss = s;
 	s += mb_len;
-	wc = MB2WC(ss, s, encode);
+	code = ENC_MBC_TO_CODE(encode, ss, s);
 
 #ifdef UNALIGNED_WORD_ACCESS
-	if (regex_is_in_wc_range(p, wc)) goto fail;
+	if (regex_is_in_code_range(p, code)) goto fail;
 #else
 	q = p;
 	ALIGNMENT_RIGHT(q);
-	if (regex_is_in_wc_range(q, wc)) goto fail;
+	if (regex_is_in_code_range(q, code)) goto fail;
 #endif
       }
       p += tlen;
@@ -1353,7 +1369,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
     case OP_CCLASS_MIX_NOT:  STAT_OP_IN(OP_CCLASS_MIX_NOT);
       DATA_ENSURE(1);
-      if (ismb(encode, *s)) {
+      if (ENC_IS_MBC_HEAD(encode, *s)) {
 	p += SIZE_BITSET;
 	goto cclass_mb_not;
       }
@@ -1370,7 +1386,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       break;
 
     case OP_ANYCHAR:  STAT_OP_IN(OP_ANYCHAR);
-      n = mblen(encode, *s);
+      n = enc_len(encode, *s);
       if (n > 1) {
 	DATA_ENSURE(n);
 	s += n;
@@ -1384,7 +1400,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       break;
 
     case OP_ANYCHAR_ML:  STAT_OP_IN(OP_ANYCHAR_ML);
-      n = mblen(encode, *s);
+      n = enc_len(encode, *s);
       DATA_ENSURE(n);
       s += n;
       STAT_OP_OUT;
@@ -1393,7 +1409,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
     case OP_ANYCHAR_STAR:  STAT_OP_IN(OP_ANYCHAR_STAR);
       while (s < end) {
 	STACK_PUSH_ALT(p, s, sprev);
-	n = mblen(encode, *s);
+	n = enc_len(encode, *s);
 	if (n > 1) {
 	  DATA_ENSURE(n);
 	  sprev = s;
@@ -1411,7 +1427,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
     case OP_ANYCHAR_ML_STAR:  STAT_OP_IN(OP_ANYCHAR_ML_STAR);
       while (s < end) {
 	STACK_PUSH_ALT(p, s, sprev);
-	n = mblen(encode, *s);
+	n = enc_len(encode, *s);
 	if (n > 1) {
 	  DATA_ENSURE(n);
 	  sprev = s;
@@ -1430,7 +1446,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	if (*p == *s) {
 	  STACK_PUSH_ALT(p + 1, s, sprev);
 	}
-	n = mblen(encode, *s);
+	n = enc_len(encode, *s);
 	if (n > 1) {
 	  DATA_ENSURE(n);
 	  sprev = s;
@@ -1451,7 +1467,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	if (*p == *s) {
 	  STACK_PUSH_ALT(p + 1, s, sprev);
 	}
-	n = mblen(encode, *s);
+	n = enc_len(encode, *s);
 	if (n >1) {
 	  DATA_ENSURE(n);
 	  sprev = s;
@@ -1468,51 +1484,35 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
     case OP_WORD:  STAT_OP_IN(OP_WORD);
       DATA_ENSURE(1);
-      if (! IS_WORD_STR_INC(encode, s, end))
+      if (! ENC_IS_MBC_E_WORD(encode, s, end))
 	goto fail;
+
+      s += enc_len(encode, *s);
       STAT_OP_OUT;
       break;
 
     case OP_NOT_WORD:  STAT_OP_IN(OP_NOT_WORD);
       DATA_ENSURE(1);
-      if (IS_WORD_STR_INC(encode, s, end))
-	goto fail;
-      STAT_OP_OUT;
-      break;
-
-#ifdef USE_SBMB_CLASS
-    case OP_WORD_SB:  STAT_OP_IN(OP_WORD_SB);
-      DATA_ENSURE(1);
-      if (! IS_SB_WORD(encode, *s))
-	goto fail;
-      s++;
-      STAT_OP_OUT;
-      break;
-
-    case OP_WORD_MB:  STAT_OP_IN(OP_WORD_MB);
-      DATA_ENSURE(1);
-      if (! IS_MB_WORD(encode, *s))
+      if (ENC_IS_MBC_E_WORD(encode, s, end))
 	goto fail;
 
-      n = mblen(encode, *s);
-      DATA_ENSURE(n);
-      s += n;
+      s += enc_len(encode, *s);
       STAT_OP_OUT;
       break;
-#endif
 
     case OP_WORD_BOUND:  STAT_OP_IN(OP_WORD_BOUND);
       if (ON_STR_BEGIN(s)) {
 	DATA_ENSURE(1);
-	if (! IS_WORD_STR(encode, s, end))
+	if (! ENC_IS_MBC_E_WORD(encode, s, end))
 	  goto fail;
       }
       else if (ON_STR_END(s)) {
-	if (! IS_WORD_STR(encode, sprev, end))
+	if (! ENC_IS_MBC_E_WORD(encode, sprev, end))
 	  goto fail;
       }
       else {
-	if (IS_WORD_STR(encode, s, end) == IS_WORD_STR(encode, sprev, end))
+	if (ENC_IS_MBC_E_WORD(encode, s, end)
+	    == ENC_IS_MBC_E_WORD(encode, sprev, end))
 	  goto fail;
       }
       STAT_OP_OUT;
@@ -1521,15 +1521,16 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
     case OP_NOT_WORD_BOUND:  STAT_OP_IN(OP_NOT_WORD_BOUND);
       if (ON_STR_BEGIN(s)) {
-	if (DATA_ENSURE_CHECK(1) && IS_WORD_STR(encode, s, end))
+	if (DATA_ENSURE_CHECK(1) && ENC_IS_MBC_E_WORD(encode, s, end))
 	  goto fail;
       }
       else if (ON_STR_END(s)) {
-	if (IS_WORD_STR(encode, sprev, end))
+	if (ENC_IS_MBC_E_WORD(encode, sprev, end))
 	  goto fail;
       }
       else {
-	if (IS_WORD_STR(encode, s, end) != IS_WORD_STR(encode, sprev, end))
+	if (ENC_IS_MBC_E_WORD(encode, s, end)
+	    != ENC_IS_MBC_E_WORD(encode, sprev, end))
 	  goto fail;
       }
       STAT_OP_OUT;
@@ -1538,8 +1539,8 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
 #ifdef USE_WORD_BEGIN_END
     case OP_WORD_BEGIN:  STAT_OP_IN(OP_WORD_BEGIN);
-      if (DATA_ENSURE_CHECK(1) && IS_WORD_STR(encode, s, end)) {
-	if (ON_STR_BEGIN(s) || !IS_WORD_STR(encode, sprev, end)) {
+      if (DATA_ENSURE_CHECK(1) && ENC_IS_MBC_E_WORD(encode, s, end)) {
+	if (ON_STR_BEGIN(s) || !ENC_IS_MBC_E_WORD(encode, sprev, end)) {
 	  STAT_OP_OUT;
 	  continue;
 	}
@@ -1548,8 +1549,8 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       break;
 
     case OP_WORD_END:  STAT_OP_IN(OP_WORD_END);
-      if (!ON_STR_BEGIN(s) && IS_WORD_STR(encode, sprev, end)) {
-	if (ON_STR_END(s) || !IS_WORD_STR(encode, s, end)) {
+      if (!ON_STR_BEGIN(s) && ENC_IS_MBC_E_WORD(encode, sprev, end)) {
+	if (ON_STR_END(s) || !ENC_IS_MBC_E_WORD(encode, s, end)) {
 	  STAT_OP_OUT;
 	  continue;
 	}
@@ -1720,7 +1721,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	DATA_ENSURE(n);
 	sprev = s;
 	STRING_CMP(pstart, s, n);
-	while (sprev + (len = mblen(encode, *sprev)) < s)
+	while (sprev + (len = enc_len(encode, *sprev)) < s)
 	  sprev += len;
 
 	STAT_OP_OUT;
@@ -1751,8 +1752,8 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	n = pend - pstart;
 	DATA_ENSURE(n);
 	sprev = s;
-	STRING_CMP_IC(pstart, s, n);
-	while (sprev + (len = mblen(encode, *sprev)) < s)
+	STRING_CMP_IC(pstart, &s, n);
+	while (sprev + (len = enc_len(encode, *sprev)) < s)
 	  sprev += len;
 
 	STAT_OP_OUT;
@@ -1787,7 +1788,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	  STRING_CMP_VALUE(pstart, swork, n, is_fail);
 	  if (is_fail) continue;
 	  s = swork;
-	  while (sprev + (len = mblen(encode, *sprev)) < s)
+	  while (sprev + (len = enc_len(encode, *sprev)) < s)
 	    sprev += len;
 
 	  p += (SIZE_MEMNUM * (tlen - i - 1));
@@ -1823,10 +1824,10 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 	  DATA_ENSURE(n);
 	  sprev = s;
 	  swork = s;
-	  STRING_CMP_VALUE_IC(pstart, swork, n, is_fail);
+	  STRING_CMP_VALUE_IC(pstart, &swork, n, is_fail);
 	  if (is_fail) continue;
 	  s = swork;
-	  while (sprev + (len = mblen(encode, *sprev)) < s)
+	  while (sprev + (len = enc_len(encode, *sprev)) < s)
 	    sprev += len;
 
 	  p += (SIZE_MEMNUM * (tlen - i - 1));
@@ -2084,9 +2085,9 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 
     case OP_LOOK_BEHIND:  STAT_OP_IN(OP_LOOK_BEHIND);
       GET_LENGTH_INC(tlen, p);
-      s = MBBACK(encode, str, s, (int )tlen);
+      s = ENC_BACK(encode, str, s, (int )tlen);
       if (IS_NULL(s)) goto fail;
-      sprev = regex_get_prev_char_head(encode, str, s);
+      sprev = enc_get_prev_char_head(encode, str, s);
       STAT_OP_OUT;
       continue;
       break;
@@ -2094,7 +2095,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
     case OP_PUSH_LOOK_BEHIND_NOT:  STAT_OP_IN(OP_PUSH_LOOK_BEHIND_NOT);
       GET_RELADDR_INC(addr, p);
       GET_LENGTH_INC(tlen, p);
-      q = MBBACK(encode, str, s, (int )tlen);
+      q = ENC_BACK(encode, str, s, (int )tlen);
       if (IS_NULL(q)) {
 	/* too short case -> success. ex. /(?<!XXX)a/.match("a")
 	   If you want to change to fail, replace following line. */
@@ -2104,7 +2105,7 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
       else {
 	STACK_PUSH_LOOK_BEHIND_NOT(p + addr, s, sprev);
 	s = q;
-	sprev = regex_get_prev_char_head(encode, str, s);
+	sprev = enc_get_prev_char_head(encode, str, s);
       }
       STAT_OP_OUT;
       continue;
@@ -2175,168 +2176,8 @@ match_at(regex_t* reg, UChar* str, UChar* end, UChar* sstart,
 }
 
 
-UChar*   DefaultTransTable = (UChar* )0;
-
-#ifndef REG_RUBY_M17N
-static const char SJIS_FOLLOW_TABLE[SINGLE_BYTE_SIZE] = {
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0
-};
-
-#define eucjp_islead(c)    ((UChar )((c) - 0xa1) > 0xfe - 0xa1)
-#define utf8_islead(c)     ((UChar )((c) & 0xc0) != 0x80)
-#define sjis_ismbfirst(c)  ismb(REGCODE_SJIS, (c))
-#define sjis_ismbtrail(c)  SJIS_FOLLOW_TABLE[(c)]
-
-extern WCINT
-regex_mb2wc(UChar* p, UChar* end, RegCharEncoding code)
-{
-  int c, i, len;
-  WCINT n;
-
-  if (code == REGCODE_UTF8) {
-    c = *p++;
-    len = mblen(code,c);
-    if (len > 1) {
-      len--;
-      n = c & ((1 << (6 - len)) - 1);
-      while (len--) {
-	c = *p++;
-	n = (n << 6) | (c & ((1 << 6) - 1));
-      }
-    }
-    else
-      n = c;
-  }
-  else {
-    c = *p++;
-    len = mblen(code,c);
-    n = c;
-    if (len == 1) return n;
-
-    for (i = 1; i < len; i++) {
-      if (p >= end) break;
-      c = *p++;
-      n <<= 8;  n += c;
-    }
-  }
-  return n;
-}
-#endif /* REG_RUBY_M17N */
-
-extern UChar*
-regex_get_left_adjust_char_head(RegCharEncoding code, UChar* start, UChar* s)
-{
-  UChar *p;
-  int len;
-
-  if (s <= start) return s;
-  p = s;
-
-#ifdef REG_RUBY_M17N
-  while (!m17n_islead(code, *p) && p > start) p--;
-  while (p + (len = mblen(code, *p)) < s) {
-    p += len;
-  }
-  if (p + len == s) return s;
-  return p;
-#else
-
-  if (code == REGCODE_ASCII) {
-    return p;
-  }
-  else if (code == REGCODE_EUCJP) {
-    while (!eucjp_islead(*p) && p > start) p--;
-    len = mblen(code, *p);
-    if (p + len > s) return p;
-    p += len;
-    return p + ((s - p) & ~1);
-  }
-  else if (code == REGCODE_SJIS) {
-    if (sjis_ismbtrail(*p)) {
-      while (p > start) {
-	if (! sjis_ismbfirst(*--p)) {
-	  p++;
-	  break;
-	}
-      } 
-    }
-    len = mblen(code, *p);
-    if (p + len > s) return p;
-    p += len;
-    return p + ((s - p) & ~1);
-  }
-  else { /* REGCODE_UTF8 */
-    while (!utf8_islead(*p) && p > start) p--;
-    return p;
-  }
-#endif  /* REG_RUBY_M17N */
-}
-
-extern UChar*
-regex_get_right_adjust_char_head(RegCharEncoding code, UChar* start, UChar* s)
-{
-  UChar* p = regex_get_left_adjust_char_head(code, start, s);
-
-  if (p < s) {
-    p += mblen(code, *p);
-  }
-  return p;
-}
-
 static UChar*
-get_right_adjust_char_head_with_prev(RegCharEncoding code,
-				     UChar* start, UChar* s, UChar** prev)
-{
-  UChar* p = regex_get_left_adjust_char_head(code, start, s);
-
-  if (p < s) {
-    if (prev) *prev = p;
-    p += mblen(code, *p);
-  }
-  else {
-    if (prev) *prev = (UChar* )NULL; /* Sorry */
-  }
-  return p;
-}
-
-extern UChar*
-regex_get_prev_char_head(RegCharEncoding code, UChar* start, UChar* s)
-{
-  if (s <= start)
-    return (UChar* )NULL;
-
-  return regex_get_left_adjust_char_head(code, start, s - 1);
-}
-
-static UChar*
-step_backward_char(RegCharEncoding code, UChar* start, UChar* s, int n)
-{
-  while (IS_NOT_NULL(s) && n-- > 0) {
-    if (s <= start)
-      return (UChar* )NULL;
-
-    s = regex_get_left_adjust_char_head(code, start, s - 1);
-  }
-  return s;
-}
-
-static UChar*
-slow_search(RegCharEncoding code, UChar* target, UChar* target_end,
+slow_search(RegCharEncoding enc, UChar* target, UChar* target_end,
 	    UChar* text, UChar* text_end, UChar* text_range)
 {
   UChar *t, *p, *s, *end;
@@ -2359,14 +2200,15 @@ slow_search(RegCharEncoding code, UChar* target, UChar* target_end,
       if (t == target_end)
 	return s;
     }
-    s += mblen(code, *s);
+    s += enc_len(enc, *s);
   }
 
   return (UChar* )NULL;
 }
 
+#if 0
 static int
-str_trans_match_after_head_byte(RegCharEncoding code,
+str_trans_match_after_head_byte(RegCharEncoding enc,
 				int len, UChar* t, UChar* tend, UChar* p)
 {
   while (--len > 0) {
@@ -2375,23 +2217,19 @@ str_trans_match_after_head_byte(RegCharEncoding code,
   }
 
   if (len == 0) {
+    int lowlen;
+    UChar *q, lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
+
     while (t < tend) {
-      len = mblen(code, *p);
-      if (len == 1) {
-	if (*t != TOLOWER(code, *p))
-	  break;
-	p++;
-	t++;
-      }
-      else {
-	if (*t != *p++)  break;
-	t++;
-	while (--len > 0) {
-	  if (*t != *p) break;
-	  t++; p++;
-	}
-	if (len > 0) break;
+      len = enc_len(enc, *p);
+      lowlen = ENC_MBC_TO_LOWER(enc, p, lowbuf);
+      q = lowbuf;
+      while (lowlen > 0) {
+	if (*t++ != *q++) break;
+	lowlen--;
       }
+      if (lowlen > 0) break;
+      p += len;
     }
     if (t == tend)
       return 1;
@@ -2399,14 +2237,36 @@ str_trans_match_after_head_byte(RegCharEncoding code,
 
   return 0;
 }
+#endif
+
+static int
+str_lower_case_match(RegCharEncoding enc, UChar* t, UChar* tend, UChar* p)
+{
+  int len, lowlen;
+  UChar *q, lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
+
+  while (t < tend) {
+    len = enc_len(enc, *p);
+    lowlen = ENC_MBC_TO_LOWER(enc, p, lowbuf);
+    q = lowbuf;
+    while (lowlen > 0) {
+      if (*t++ != *q++) return 0;
+      lowlen--;
+    }
+    p += len;
+  }
+
+  return 1;
+}
 
 static UChar*
-slow_search_ic(RegCharEncoding code,
+slow_search_ic(RegCharEncoding enc,
 	       UChar* target, UChar* target_end,
 	       UChar* text, UChar* text_end, UChar* text_range)
 {
-  int len;
+  int len, lowlen;
   UChar *t, *p, *s, *end;
+  UChar lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
 
   end = text_end - (target_end - target) + 1;
   if (end > text_range)
@@ -2415,13 +2275,21 @@ slow_search_ic(RegCharEncoding code,
   s = text;
 
   while (s < end) {
-    len = mblen(code, *s);
-    if (*s == *target || (len == 1 && TOLOWER(code, *s) == *target)) {
-      p = s + 1;
+    len = enc_len(enc, *s);
+    lowlen = ENC_MBC_TO_LOWER(enc, s, lowbuf);
+    if (*target == *lowbuf) {
+      p = lowbuf + 1;
       t = target + 1;
-      if (str_trans_match_after_head_byte(code, len, t, target_end, p))
-	return s;
+      while (--lowlen > 0) {
+	if (*p != *t) break;
+	p++; *t++;
+      }
+      if (lowlen == 0) {
+	if (str_lower_case_match(enc, t, target_end, s + len))
+	  return s;
+      }
     }
+
     s += len;
   }
 
@@ -2429,7 +2297,7 @@ slow_search_ic(RegCharEncoding code,
 }
 
 static UChar*
-slow_search_backward(RegCharEncoding code, UChar* target, UChar* target_end,
+slow_search_backward(RegCharEncoding enc, UChar* target, UChar* target_end,
 	     UChar* text, UChar* adjust_text, UChar* text_end, UChar* text_start)
 {
   UChar *t, *p, *s;
@@ -2438,7 +2306,7 @@ slow_search_backward(RegCharEncoding code, UChar* target, UChar* target_end,
   if (s > text_start)
     s = text_start;
   else
-    s = regex_get_left_adjust_char_head(code, adjust_text, s);
+    s = ENC_LEFT_ADJUST_CHAR_HEAD(enc, adjust_text, s);
 
   while (s >= text) {
     if (*s == *target) {
@@ -2452,36 +2320,45 @@ slow_search_backward(RegCharEncoding code, UChar* target, UChar* target_end,
       if (t == target_end)
 	return s;
     }
-    s = regex_get_prev_char_head(code, adjust_text, s);
+    s = enc_get_prev_char_head(enc, adjust_text, s);
   }
 
   return (UChar* )NULL;
 }
 
 static UChar*
-slow_search_backward_ic(RegCharEncoding code,
+slow_search_backward_ic(RegCharEncoding enc,
 			UChar* target,UChar* target_end,
 			UChar* text, UChar* adjust_text,
 			UChar* text_end, UChar* text_start)
 {
-  int len;
+  int len, lowlen;
   UChar *t, *p, *s;
+  UChar lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
 
   s = text_end - (target_end - target);
   if (s > text_start)
     s = text_start;
   else
-    s = regex_get_left_adjust_char_head(code, adjust_text, s);
+    s = ENC_LEFT_ADJUST_CHAR_HEAD(enc, adjust_text, s);
 
   while (s >= text) {
-    len = mblen(code, *s);
-    if (*s == *target || (len == 1 && TOLOWER(code, *s) == *target)) {
-      p = s + 1;
+    len = enc_len(enc, *s);
+    lowlen = ENC_MBC_TO_LOWER(enc, s, lowbuf);
+    if (*target == *lowbuf) {
+      p = lowbuf + 1;
       t = target + 1;
-      if (str_trans_match_after_head_byte(code, len, t, target_end, p))
-	return s;
+      while (--lowlen > 0) {
+	if (*p != *t) break;
+	p++; *t++;
+      }
+      if (lowlen == 0) {
+	if (str_lower_case_match(enc, t, target_end, s + len))
+	  return s;
+      }
     }
-    s = regex_get_prev_char_head(code, adjust_text, s);
+
+    s = enc_get_prev_char_head(enc, adjust_text, s);
   }
 
   return (UChar* )NULL;
@@ -2502,7 +2379,7 @@ bm_search_notrev(regex_t* reg, UChar* target, UChar* target_end,
   tail = target_end - 1;
   s = text;
   while ((s - text) < target_end - target) {
-    s += mblen(reg->enc, *s);
+    s += enc_len(reg->enc, *s);
   }
   s--; /* set to text check tail position. */
 
@@ -2519,7 +2396,7 @@ bm_search_notrev(regex_t* reg, UChar* target, UChar* target_end,
       p++;
       t = p;
       while ((p - t) < skip) {
-	p += mblen(reg->enc, *p);
+	p += enc_len(reg->enc, *p);
       }
       s += (p - t);
     }
@@ -2537,7 +2414,7 @@ bm_search_notrev(regex_t* reg, UChar* target, UChar* target_end,
       p++;
       t = p;
       while ((p - t) < skip) {
-	p += mblen(reg->enc, *p);
+	p += enc_len(reg->enc, *p);
       }
       s += (p - t);
     }
@@ -2588,6 +2465,7 @@ set_bm_backward_skip(UChar* s, UChar* end, RegCharEncoding enc,
 		     int ignore_case, int** skip)
 {
   int i, len;
+  UChar lowbuf[ENC_MBC_TO_LOWER_MAXLEN];
 
   if (IS_NULL(*skip)) {
     *skip = (int* )xmalloc(sizeof(int) * REG_CHAR_TABLE_SIZE);
@@ -2599,8 +2477,10 @@ set_bm_backward_skip(UChar* s, UChar* end, RegCharEncoding enc,
     (*skip)[i] = len;
 
   if (ignore_case) {
-    for (i = len - 1; i > 0; i--)
-      (*skip)[TOLOWER(enc, s[i])] = i;
+    for (i = len - 1; i > 0; i--) {
+      ENC_MBC_TO_LOWER(enc, &(s[i]), lowbuf);
+      (*skip)[*lowbuf] = i;
+    }
   }
   else {
     for (i = len - 1; i > 0; i--)
@@ -2619,7 +2499,7 @@ bm_search_backward(regex_t* reg, UChar* target, UChar* target_end, UChar* text,
   if (text_start < s)
     s = text_start;
   else
-    s = regex_get_left_adjust_char_head(reg->enc, adjust_text, s);
+    s = ENC_LEFT_ADJUST_CHAR_HEAD(reg->enc, adjust_text, s);
 
   while (s >= text) {
     p = s;
@@ -2631,27 +2511,27 @@ bm_search_backward(regex_t* reg, UChar* target, UChar* target_end, UChar* text,
       return s;
 
     s -= reg->int_map_backward[*s];
-    s = regex_get_left_adjust_char_head(reg->enc, adjust_text, s);
+    s = ENC_LEFT_ADJUST_CHAR_HEAD(reg->enc, adjust_text, s);
   }
 
   return (UChar* )NULL;
 }
 
 static UChar*
-map_search(RegCharEncoding code, UChar map[], UChar* text, UChar* text_range)
+map_search(RegCharEncoding enc, UChar map[], UChar* text, UChar* text_range)
 {
   UChar *s = text;
 
   while (s < text_range) {
     if (map[*s]) return s;
 
-    s += mblen(code, *s);
+    s += enc_len(enc, *s);
   }
   return (UChar* )NULL;
 }
 
 static UChar*
-map_search_backward(RegCharEncoding code, UChar map[],
+map_search_backward(RegCharEncoding enc, UChar map[],
 		    UChar* text, UChar* adjust_text, UChar* text_start)
 {
   UChar *s = text_start;
@@ -2659,7 +2539,7 @@ map_search_backward(RegCharEncoding code, UChar map[],
   while (s >= text) {
     if (map[*s]) return s;
 
-    s = regex_get_prev_char_head(code, adjust_text, s);
+    s = enc_get_prev_char_head(enc, adjust_text, s);
   }
   return (UChar* )NULL;
 }
@@ -2674,13 +2554,18 @@ regex_match(regex_t* reg, UChar* str, UChar* end, UChar* at, RegRegion* region,
 
   MATCH_ARG_INIT(msa, option, region, at);
 
-  if (region && !IS_POSIX_REGION(option))
+  if (region
+#ifdef USE_POSIX_REGION_OPTION
+      && !IS_POSIX_REGION(option)
+#endif
+      ) {
     r = regex_region_resize(region, reg->num_mem + 1);
+  }
   else
     r = 0;
 
   if (r == 0) {
-    prev = regex_get_prev_char_head(reg->enc, str, at);
+    prev = enc_get_prev_char_head(reg->enc, str, at);
     r = match_at(reg, str, end, at, prev, &msa);
   }
   MATCH_ARG_FREE(msa);
@@ -2700,12 +2585,12 @@ forward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
 
   p = s;
   if (reg->dmin > 0) {
-    if (IS_SINGLEBYTE_CODE(reg->enc)) {
+    if (ENC_IS_SINGLEBYTE(reg->enc)) {
       p += reg->dmin;
     }
     else {
       UChar *q = p + reg->dmin;
-      while (p < q) p += mblen(reg->enc, *p);
+      while (p < q) p += enc_len(reg->enc, *p);
     }
   }
 
@@ -2735,7 +2620,7 @@ forward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
     if (p - reg->dmin < s) {
     retry_gate:
       pprev = p;
-      p += mblen(reg->enc, *p);
+      p += enc_len(reg->enc, *p);
       goto retry;
     }
 
@@ -2745,7 +2630,7 @@ forward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
       switch (reg->sub_anchor) {
       case ANCHOR_BEGIN_LINE:
 	if (!ON_STR_BEGIN(p)) {
-	  prev = regex_get_prev_char_head(reg->enc, (pprev ? pprev : str), p);
+	  prev = enc_get_prev_char_head(reg->enc, (pprev ? pprev : str), p);
 	  if (!IS_NEWLINE(*prev))
 	    goto retry_gate;
 	}
@@ -2753,7 +2638,7 @@ forward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
 
       case ANCHOR_END_LINE:
 	if (ON_STR_END(p)) {
-	  prev = regex_get_prev_char_head(reg->enc, (pprev ? pprev : str), p);
+	  prev = enc_get_prev_char_head(reg->enc, (pprev ? pprev : str), p);
 	  if (prev && IS_NEWLINE(*prev))
 	    goto retry_gate;
 	}
@@ -2767,26 +2652,26 @@ forward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
       *low = p;
       if (low_prev) {
 	if (*low > s)
-	  *low_prev = regex_get_prev_char_head(reg->enc, s, p);
+	  *low_prev = enc_get_prev_char_head(reg->enc, s, p);
 	else
-	  *low_prev = regex_get_prev_char_head(reg->enc,
-					       (pprev ? pprev : str), p);
+	  *low_prev = enc_get_prev_char_head(reg->enc,
+					     (pprev ? pprev : str), p);
       }
     }
     else {
       if (reg->dmax != INFINITE_DISTANCE) {
 	*low = p - reg->dmax;
 	if (*low > s) {
-	  *low = get_right_adjust_char_head_with_prev(reg->enc, s,
-						      *low, low_prev);
+	  *low = enc_get_right_adjust_char_head_with_prev(reg->enc, s,
+							  *low, low_prev);
 	  if (low_prev && IS_NULL(*low_prev))
-	    *low_prev = regex_get_prev_char_head(reg->enc,
-						 (pprev ? pprev : s), *low);
+	    *low_prev = enc_get_prev_char_head(reg->enc,
+					       (pprev ? pprev : s), *low);
 	}
 	else {
 	  if (low_prev)
-	    *low_prev = regex_get_prev_char_head(reg->enc,
-						 (pprev ? pprev : str), *low);
+	    *low_prev = enc_get_prev_char_head(reg->enc,
+					       (pprev ? pprev : str), *low);
 	}
       }
     }
@@ -2858,7 +2743,7 @@ backward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
       switch (reg->sub_anchor) {
       case ANCHOR_BEGIN_LINE:
 	if (!ON_STR_BEGIN(p)) {
-	  prev = regex_get_prev_char_head(reg->enc, adjrange, p);
+	  prev = enc_get_prev_char_head(reg->enc, adjrange, p);
 	  if (!IS_NEWLINE(*prev)) {
 	    p = prev;
 	    goto retry;
@@ -2868,7 +2753,7 @@ backward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
 
       case ANCHOR_END_LINE:
 	if (ON_STR_END(p)) {
-	  prev = regex_get_prev_char_head(reg->enc, adjrange, p);
+	  prev = enc_get_prev_char_head(reg->enc, adjrange, p);
 	  if (IS_NULL(prev)) goto fail;
 	  if (IS_NEWLINE(*prev)) {
 	    p = prev;
@@ -2876,7 +2761,7 @@ backward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
 	  }
 	}
 	else if (!IS_NEWLINE(*p)) {
-	  p = regex_get_prev_char_head(reg->enc, adjrange, p);
+	  p = enc_get_prev_char_head(reg->enc, adjrange, p);
 	  if (IS_NULL(p)) goto fail;
 	  goto retry;
 	}
@@ -2888,7 +2773,7 @@ backward_search_range(regex_t* reg, UChar* str, UChar* end, UChar* s,
     if (reg->dmax != INFINITE_DISTANCE) {
       *low  = p - reg->dmax;
       *high = p - reg->dmin;
-      *high = regex_get_right_adjust_char_head(reg->enc, adjrange, *high);
+      *high = enc_get_right_adjust_char_head(reg->enc, adjrange, *high);
     }
 
 #ifdef REG_DEBUG_SEARCH
@@ -2936,7 +2821,11 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	  (int )str, (int )(end - str), (int )(start - str), (int )(range - str));
 #endif
 
-  if (region && !IS_POSIX_REGION(option)) {
+  if (region
+#ifdef USE_POSIX_REGION_OPTION
+      && !IS_POSIX_REGION(option)
+#endif
+      ) {
     r = regex_region_resize(region, reg->num_mem + 1);
     if (r) goto finish_no_msa;
   }
@@ -2988,9 +2877,9 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	if (semi_end - start > reg->anchor_dmax) {
 	  start = semi_end - reg->anchor_dmax;
 	  if (start < end)
-	    start = regex_get_right_adjust_char_head(reg->enc, str, start);
+	    start = enc_get_right_adjust_char_head(reg->enc, str, start);
 	  else { /* match with empty at end */
-	    start = regex_get_prev_char_head(reg->enc, str, end);
+	    start = enc_get_prev_char_head(reg->enc, str, end);
 	  }
 	}
 	if (semi_end - (range - 1) < reg->anchor_dmin) {
@@ -3005,7 +2894,7 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	}
 	if (semi_end - start < reg->anchor_dmin) {
 	  start = semi_end - reg->anchor_dmin;
-	  start = regex_get_left_adjust_char_head(reg->enc, str, start);
+	  start = ENC_LEFT_ADJUST_CHAR_HEAD(reg->enc, str, start);
 	  if (range > start) goto mismatch_no_msa;
 	}
       }
@@ -3054,7 +2943,7 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
   s = start;
   if (range > start) {   /* forward search */
     if (s > str)
-      prev = regex_get_prev_char_head(reg->enc, str, s);
+      prev = enc_get_prev_char_head(reg->enc, str, s);
     else
       prev = (UChar* )NULL;
 
@@ -3082,13 +2971,13 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	  while (s <= high) {
 	    MATCH_AND_RETURN_CHECK;
 	    prev = s;
-	    s += mblen(reg->enc, *s);
+	    s += enc_len(reg->enc, *s);
 	  }
 	  if ((reg->anchor & ANCHOR_ANYCHAR_STAR) != 0) {
 	    if (IS_NOT_NULL(prev)) {
 	      while (!IS_NEWLINE(*prev) && s < range) {
 		prev = s;
-		s += mblen(reg->enc, *s);
+		s += enc_len(reg->enc, *s);
 	      }
 	    }
 	  }
@@ -3105,14 +2994,14 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
     do {
       MATCH_AND_RETURN_CHECK;
       prev = s;
-      s += mblen(reg->enc, *s);
+      s += enc_len(reg->enc, *s);
     } while (s <= range);   /* exec s == range, because empty match with /$/. */
   }
   else {  /* backward search */
     if (reg->optimize != REG_OPTIMIZE_NONE) {
       UChar *low, *high, *adjrange, *sch_start;
 
-      adjrange = regex_get_left_adjust_char_head(reg->enc, str, range);
+      adjrange = ENC_LEFT_ADJUST_CHAR_HEAD(reg->enc, str, range);
       if (reg->dmax != INFINITE_DISTANCE &&
 	  (end - range) >= reg->threshold_len) {
 	do {
@@ -3126,7 +3015,7 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	    s = high;
 
 	  while (s >= low) {
-	    prev = regex_get_prev_char_head(reg->enc, str, s);
+	    prev = enc_get_prev_char_head(reg->enc, str, s);
 	    MATCH_AND_RETURN_CHECK;
 	    s = prev;
 	  }
@@ -3144,8 +3033,8 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 	    sch_start += reg->dmax;
 	    if (sch_start > end) sch_start = end;
 	    else
-	      sch_start = regex_get_left_adjust_char_head(reg->enc, start,
-							  sch_start);
+	      sch_start = ENC_LEFT_ADJUST_CHAR_HEAD(reg->enc,
+						    start, sch_start);
 	  }
 	}
 	if (backward_search_range(reg, str, end, sch_start, range, adjrange,
@@ -3154,7 +3043,7 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
     }
 
     do {
-      prev = regex_get_prev_char_head(reg->enc, str, s);
+      prev = enc_get_prev_char_head(reg->enc, str, s);
       MATCH_AND_RETURN_CHECK;
       s = prev;
     } while (s >= range);
@@ -3169,8 +3058,13 @@ regex_search(regex_t* reg, UChar* str, UChar* end,
 
   /* If result is mismatch and no FIND_NOT_EMPTY option,
      then the region is not setted in match_at(). */
-  if (IS_FIND_NOT_EMPTY(reg->options) && region && !IS_POSIX_REGION(option))
+  if (IS_FIND_NOT_EMPTY(reg->options) && region
+#ifdef USE_POSIX_REGION_OPTION
+      && !IS_POSIX_REGION(option)
+#endif
+      ) {
     regex_region_clear(region);
+  }
 
 #ifdef REG_DEBUG
   if (r != REG_MISMATCH)
