@@ -27,7 +27,7 @@
 
 #if defined(__i386) || defined(__i386__) || defined(_M_IX86) || \
     (defined(__ppc__) && defined(__APPLE__)) || \
-    defined(__mc68020__)
+    defined(__mc68020__) || defined(__x86_64) || defined(__x86_64__)
 #define UNALIGNED_WORD_ACCESS
 #endif
 
